import numpy as np        
import copy
import pandas as pd
from orbit import timeperi_to_timetrans, timetrans_to_timeperi

# List of available bases
BASIS_NAMES = [
'per tp e w k', # The CPS basis
'per tc secosw sesinw logk',
'per tc secosw sesinw k',
'per tc e w k'

]
    
def _print_valid_basis():
    print "Available bases:"
    print "\n".join(BASIS_NAMES)

class Basis(object):
    """
    Object that knows how to convert between the various Keplerian bases

    Args:
        name (str): basis name
        num_planets (int): number of planets

    Attributes:
        cps_params (str): name of CPS basis

    Note:
        Valid basis functions: \n
        'per tp e w k' (The CPS basis) \n
        'per tc secosw sesinw logk'  \n 
        'per tc secosw sesinw k'  \n
        'per tc e w k'
    """
    
    cps_params = 'per tp e w k'.split()
    def __init__(self, *args):
        self.name = None
        self.num_planets = 0
        if len(args)==0:
            _print_valid_basis()
            return None
        
        name, num_planets = args

        if BASIS_NAMES.count(name)==0:
            print "{} not valid basis".format(name)
            _print_valid_basis()
            return None

        self.name = name
        self.num_planets = num_planets
        self.params = name.split()

    def __repr__(self):
        return "Basis Object <{}>".format(self.name)

        
    def to_cps(self, params_in, **kwargs):
        """Convert to CPS basis

        Convert a dictionary with parameters of a given basis into the
        cps basis

        Args:
            params_in (dict): planet parameters expressed in current basis

        Returns: 
            dict or DataFrame: parameters expressed in the CPS basis

        """

        params_out = copy.copy(params_in)
        for num_planet in range(1,1+self.num_planets):
            def _getpar(key):
                return params_out['{}{}'.format(key,num_planet)]
            def _setpar(key, value):
                params_out['{}{}'.format(key,num_planet)] = value
            def _delpar(key):
                if isinstance(params_in,dict):
                    del params_out['{}{}'.format(key,num_planet)]
                elif isinstance(params_in,pd.core.frame.DataFrame):
                    params_out.drop('{}{}'.format(key,num_planet))

            # transform into CPS basis
            if self.name == 'per tp e w k':
                # already in the CPS basis
                per = _getpar('per')
                tp = _getpar('tp')
                e = _getpar('e')
                w = _getpar('w')   
                k = _getpar('k')
                
            if self.name == 'per tc e w k':
                per = _getpar('per')
                tc = _getpar('tc')
                e = _getpar('e')
                w = _getpar('w')
                k = _getpar('k')
                tp = timetrans_to_timeperi(tc, per, e, w)
            
            if self.name=='per tc secosw sesinw logk':
                # pull out parameters
                per = _getpar('per')
                tc = _getpar('tc')
                secosw = _getpar('secosw')
                sesinw = _getpar('sesinw')
                logk = _getpar('logk')

                k = np.exp(logk)
                e = secosw**2 + sesinw**2
                w = np.arctan2(sesinw , secosw)
                tp = timetrans_to_timeperi(tc, per, e, w)

            if self.name=='per tc secosw sesinw k':
                # pull out parameters
                per = _getpar('per')
                tc = _getpar('tc')
                secosw = _getpar('secosw')
                sesinw = _getpar('sesinw')
                k = _getpar('k')
            
                # transform into CPS basis
                e = secosw**2 + sesinw**2
                w = np.arctan2(sesinw , secosw)
                tp = timetrans_to_timeperi(tc, per, e, w)

                                
            # shoves cps parameters from namespace into param_out
            _setpar('per', per)
            _setpar('tp', tp)
            _setpar('e', e)
            _setpar('w', w)
            _setpar('k', k)


        return params_out


    def from_cps(self, params_in, newbasis, **kwargs):
        """Convert from CPS basis into another basis

        Convert a dictionary with parameters of a given basis into the cps basis

        Args:
            params_in (dict):  planet parameters expressed in cps basis
            newbasis (string): string corresponding to basis to switch into
            keep (Optional[bool]): keep the parameters expressed in
                the old basis, else remove them from the output
                dictionary/DataFrame

        Returns:
            dict or dataframe with the parameters converted into the new basis
        """
        
        if newbasis not in BASIS_NAMES:
            print "{} not valid basis".format(newbasis)
            _print_valid_basis()
            return None
        
        params_out = copy.copy(params_in)
        for num_planet in range(1,1+self.num_planets):
            def _getpar(key):
                return params_out['{}{}'.format(key,num_planet)]
            def _setpar(key, value):
                params_out['{}{}'.format(key,num_planet)] = value
            def _delpar(key):
                if isinstance(params_in,dict):
                    del params_out['{}{}'.format(key,num_planet)]
                elif isinstance(params_in,pd.core.frame.DataFrame):
                    params_out.drop('{}{}'.format(key,num_planet))
            

            if newbasis == 'per tc e w k':
                per = _getpar('per')
                e = _getpar('e')
                w = _getpar('w')
                tp = _getpar('tp')
                
                _setpar('tc', timeperi_to_timetrans(tp, per, e, w) )
                _setpar('w', w )

                if not kwargs.get('keep', True):
                    _delpar('tp')

            
            if newbasis == 'per tc secosw sesinw logk':
                per = _getpar('per')
                e = _getpar('e')
                w = _getpar('w')
                k = _getpar('k')
                try:
                    tp = _getpar('tp')
                except KeyError:
                    tc = _getpar('tc')
                    tp = timetrans_to_timeperi(tc, per, e, w)
                    _setpar('tp', tp)
                    
                _setpar('secosw', np.sqrt(e)*np.cos(w) )
                _setpar('sesinw', np.sqrt(e)*np.sin(w) )
                _setpar('logk', np.log(k) )
                _setpar('tc', timeperi_to_timetrans(tp, per, e, w) )

                if not kwargs.get('keep', True):
                    _delpar('tp')
                    _delpar('e')
                    _delpar('w')
                    _delpar('k')

                self.name = newbasis
                self.params = newbasis.split()

                
            if newbasis == 'per tc secosw sesinw k':
                per = _getpar('per')
                e = _getpar('e')
                w = _getpar('w')
                k = _getpar('k')
                try:
                    tp = _getpar('tp')
                except KeyError:
                    tp = timetrans_to_timeperi(_getpar('tc'), per, e, w)
                    _setpar('tp', tp)
                _setpar('secosw', np.sqrt(e)*np.cos(w) )
                _setpar('sesinw', np.sqrt(e)*np.sin(w) )
                _setpar('k', k )
                _setpar('tc', timeperi_to_timetrans(tp, per, e, w) )

                if not kwargs.get('keep', True):
                    _delpar('tp')
                    _delpar('e')
                    _delpar('w')

                self.name = newbasis
                self.params = newbasis.split()

        return params_out
                



import numpy as np
from radvel import model

class Prior(object):
    def __repr__(self):
        return "Generic Prior"

class Gaussian(Prior):
    """Gaussian prior

    Guassian prior on a given parameter.

    Args:
        param (string): parameter label
        mu (float): center of Gaussian prior
        sigma (float): width of Gaussian prior
    """
    
    def __init__(self, param, mu, sigma):
        self.mu = mu
        self.sigma = sigma
        self.param = param
    def __call__(self, params):
        x = params[self.param]
        return -0.5 * ( ( x - self.mu ) / self.sigma )**2
    def __repr__(self):
        s = "Gaussian prior on {}, mu={}, sigma={}".format(
            self.param, self. mu, self.sigma
            )
        return s
    def __str__(self):
        try:
            d = {self.param: self.mu}
            tex = model.RVParameters(9).tex_labels(param_list=[self.param])[self.param]
            
            s = "Gaussian prior on {}: ${} \\pm {}$ \\\\".format(tex, self. mu, self.sigma)
        except KeyError:
            s = self.__repr__()
            
        return s

class EccentricityPrior(Prior):
    """Physical eccentricities

    Prior to keep eccentricity between 0 and a specified upper limit.

    Args:
        num_planets (int or list): Planets to apply the eccentricity prior.
            If an integer is given then all planets with indexes up to and including
            the specified integer will be included in the prior. If a list is given then
            the prior will only be applied to the specified planets.
        upperlims (float or list of floats): List of eccentricity upper limits to assign
            to each of the planets. If a float is given then all planets must have
            eccentricities less then this value. If a list of floats is given then
            each planet can have a different eccentricity upper limit.
    """

    
    def __repr__(self):
        msg = ""
        for i,num_planet in enumerate(self.planet_list):
            msg += "e{} constrained to be < {}\n".format(num_planet, self.upperlims[i])

        return msg[:-1]
    
    def __str__(self):
        tex = model.RVParameters(9, basis='per tc e w k').tex_labels()

        msg = ""
        for i,num_planet in enumerate(self.planet_list):
            par = "e{}".format(num_planet)
            label = tex[par]
            msg += "{} constrained to be $<{}$ \\\\\\\\\n".format(label, self.upperlims[i])

        return msg[:-5]
        
    def __init__(self, num_planets, upperlims=0.99):

        if type(num_planets) == int:
            self.planet_list = range(1,num_planets+1)
        else:
            self.planet_list = num_planets
            
        if type(upperlims) == float:
            self.upperlims = [upperlims] * len(self.planet_list)
        else:
            assert len(upperlims) == len(self.planet_list), "Number of eccentricity \
upper limits must match number of planets."
            self.upperlims = upperlims
    
    def __call__(self, params):
        def _getpar(key, num_planet):
            return params['{}{}'.format(key,num_planet)]
        
        for i,num_planet in enumerate(self.planet_list):
            try:
                ecc = _getpar('e', num_planet)
            except KeyError:
                secosw = _getpar('secosw',num_planet)
                sesinw = _getpar('sesinw',num_planet)
                ecc = secosw**2 + sesinw**2 

            if ecc > self.upperlims[i] or ecc < 0.0:
                return -np.inf
        
        return 0
        
class PositiveKPrior(Prior):
    """K must be positive

    A prior to prevent K going negative.
    Be careful with this as it can introduce a bias to larger K
    values.

    Args:
        num_planets (int): Number of planets. Used to ensure K for each
            planet is positive
    """
    
    def __repr__(self):
        return "K constrained to be > 0"
    def __str__(self):
        return "$K$ constrained to be $>0$"

    def __init__(self, num_planets):
        self.num_planets = num_planets
    
    def __call__(self, params):
        def _getpar(key, num_planet):
            return params['{}{}'.format(key,num_planet)]

        for num_planet in range(1,self.num_planets+1):
            try:
                k = _getpar('k', num_planet)
            except KeyError:
                k = np.exp(_getpar('logk',num_planet))

            if k < 0.0:    
                return -np.inf
        return 0

class HardBounds(Prior):
    """Prior for hard boundaries

    This prior allows for hard boundaries to be established
    for a given parameter.

    Args:
        param (string): parameter label
        minval (float): minimum allowed value
        maxval (float): maximum allowed value
    """
    
    def __init__(self, param, minval, maxval):
        self.minval = minval
        self.maxval = maxval
        self.param = param
    def __call__(self, params):
        x = params[self.param]
        if x < self.minval or x > self.maxval:
            return -np.inf
        else:
            return 0.0
    def __repr__(self):
        s = "Bounded prior on {}, min={}, max={}".format(
            self.param, self.minval, self.maxval
            )
        return s
    def __str__(self):
        try:
            d = {self.param: self.minval}
            tex = model.RVParameters(9).tex_labels(param_list=[self.param])[self.param]
            
            s = "Bounded prior: ${} < {} < {}$".format(self.minval,
                                                       tex.replace('$',''),
                                                       self.maxval)
        except KeyError:
            s = self.__repr__()
            
        return s



__all__=['model','likelihood','posterior','mcmc','prior', 'utils', 'fitting', 'report', 'cli', 'driver']
from .model import *
from .likelihood import *
from . import posterior
from mcmc import *
from .prior import *
from .utils import *
from .report import *
from .plotting import *
from .fitting import *
import os 
import sys

__version__ = '0.8.0'

MODULEDIR, filename = os.path.split(__file__)
DATADIR = os.path.join(sys.prefix,'radvel_example_data')



"""
Driver functions for the radvel pipeline.\
These functions are meant to be used only with\
the `cli.py` command line interface.
"""
import os
import pickle
import copy
import ConfigParser
from collections import OrderedDict
import pandas as pd

import numpy as np 

import radvel

def plots(args):
    """
    Generate plots

    Args:
        args (ArgumentParser): command line arguments
    """
    
    config_file = args.setupfn
    conf_base = os.path.basename(config_file).split('.')[0]
    statfile = os.path.join(
        args.outputdir,"{}_radvel.stat".format(conf_base)
    )

    status = load_status(statfile)

    assert status.getboolean('fit', 'run'), \
      "Must perform max-liklihood fit before plotting"
    post = radvel.posterior.load(status.get('fit', 'postfile'))

    for ptype in args.type:
        print "Creating {} plot for {}".format(ptype, conf_base)

        if ptype == 'rv':
            args.plotkw['uparams'] = post.uparams
            saveto = os.path.join(
                args.outputdir,conf_base+'_rv_multipanel.pdf'
            )
            P,_ = radvel.utils.initialize_posterior(config_file)
            if hasattr(P, 'bjd0'):
                args.plotkw['epoch'] = P.bjd0
            radvel.plotting.rv_multipanel_plot(
                post, saveplot=saveto, **args.plotkw
            )

        if ptype == 'corner' or ptype == 'trend':
            assert status.getboolean('mcmc', 'run'), \
            "Must run MCMC before making corner or trend plots"

            chains = pd.read_csv(status.get('mcmc', 'chainfile'))

        if ptype == 'corner':
            saveto = os.path.join(args.outputdir, conf_base+'_corner.pdf')
            radvel.plotting.corner_plot(post, chains, saveplot=saveto)

        if ptype == 'trend':
            nwalkers = status.getint('mcmc', 'nwalkers')

            saveto = os.path.join(args.outputdir, conf_base+'_trends.pdf')
            radvel.plotting.trend_plot(post, chains, nwalkers, saveto)

        if ptype == 'derived':
            assert status.has_section('derive'), \
            "Must run `radvel derive` before plotting derived parameters"

            P,_ = radvel.utils.initialize_posterior(config_file)
            chains = pd.read_csv(status.get('derive', 'chainfile'))
            saveto = os.path.join(
                args.outputdir,conf_base+'_corner_derived_pars.pdf'
            )
            radvel.plotting.corner_plot_derived_pars(
                chains, P, saveplot=saveto
            )

        savestate = {'{}_plot'.format(ptype): os.path.abspath(saveto)}
        save_status(statfile, 'plot', savestate)
            
        
def fit(args):
    """Perform maximum-likelihood fit

    Args:
        args (ArgumentParser): command line arguments
    """

    config_file = args.setupfn
    conf_base = os.path.basename(config_file).split('.')[0]
    print "Performing max-likelihood fitting for {}".format(conf_base)

    P, post = radvel.utils.initialize_posterior(config_file)
    post = radvel.fitting.maxlike_fitting(post, verbose=True)

    postfile = os.path.join(args.outputdir,
                            '{}_post_obj.pkl'.format(conf_base))
    post.writeto(postfile)

    savestate = {'run': True,
                 'postfile': os.path.abspath(postfile)}
    save_status(os.path.join(args.outputdir,
                             '{}_radvel.stat'.format(conf_base)),
                             'fit', savestate)
            
def mcmc(args):
    """Perform MCMC error analysis

    Args:
        args (ArgumentParser): command line arguments
    """

    config_file = args.setupfn
    conf_base = os.path.basename(config_file).split('.')[0]
    statfile = os.path.join(args.outputdir,
                            "{}_radvel.stat".format(conf_base))

    status = load_status(statfile)

    if status.getboolean('fit', 'run'):
        print "Loading starting positions from previous max-likelihood fit"

        post = radvel.posterior.load(status.get('fit', 'postfile'))
    else:
        P, post = radvel.utils.initialize_posterior(config_file)


    msg = "Running MCMC for {}, nwalkers = {}, nsteps = {} ...".format(
        conf_base, args.nwalkers, args.nsteps)
    print msg

    chains = radvel.mcmc(
        post, threads=1, nwalkers=args.nwalkers, nrun=args.nsteps
    )


    # Convert chains into CPS basis
    cpschains = chains.copy()
    for par in post.params.keys():
        if not post.vary[par]:
            cpschains[par] = post.params[par]

    cpschains = post.params.basis.to_cps(cpschains)
    cps_quantile = cpschains.quantile([0.159, 0.5, 0.841])

    # Get quantiles and update posterior object
    post_summary=chains.quantile([0.159, 0.5, 0.841])        

    for k in chains.keys():
        if k in post.params.keys():
            post.params[k] = post_summary[k][0.5]

    print "Performing post-MCMC maximum likelihood fit..."
    post = radvel.fitting.maxlike_fitting(post, verbose=False)

    cpspost = copy.deepcopy(post)
    cpsparams = post.params.basis.to_cps(post.params)
    cpspost.params.update(cpsparams)


    print "Calculating uncertainties..."
    cpspost.uparams = {}
    cpspost.medparams = {}
    cpspost.maxparams = {}
    for par in cpspost.params.keys():
        maxlike = cpspost.params[par]
        med = cps_quantile[par][0.5]
        high = cps_quantile[par][0.841] - med
        low = med - cps_quantile[par][0.159]
        err = np.mean([high,low])
        err = radvel.utils.round_sig(err)
        med, err, errhigh = radvel.utils.sigfig(med, err)
        maxlike, err, errhigh = radvel.utils.sigfig(maxlike, err)
        cpspost.uparams[par] = err
        cpspost.medparams[par] = med
        cpspost.maxparams[par] = maxlike


    print "Final loglikelihood = %f" % post.logprob()
    print "Final RMS = %f" % post.likelihood.residuals().std()
    print "Best-fit parameters:"
    print cpspost

    print "Saving output files..."
    saveto = os.path.join(args.outputdir, conf_base+'_post_summary.csv')
    post_summary.to_csv(saveto, sep=',')

    postfile = os.path.join(args.outputdir,
                            '{}_post_obj.pkl'.format(conf_base))
    cpspost.writeto(postfile)

    csvfn = os.path.join(args.outputdir, conf_base+'_chains.csv.tar.bz2')
    chains.to_csv(csvfn, compression='bz2')


    savestate = {'run': True,
                 'postfile': os.path.abspath(postfile),
                 'chainfile': os.path.abspath(csvfn),
                 'summaryfile': os.path.abspath(saveto),
                 'nwalkers': args.nwalkers,
                 'nsteps': args.nsteps}
    save_status(statfile, 'mcmc', savestate)


def bic(args):
    """Compare different models and comparative statistics

    Args:
        args (ArgumentParser): command line arguments
    """

    
    config_file = args.setupfn
    conf_base = os.path.basename(config_file).split('.')[0]
    statfile = os.path.join(args.outputdir,
                            "{}_radvel.stat".format(conf_base))


    status = load_status(statfile)
    savestate = {}

    assert status.getboolean('fit', 'run'), \
      "Must perform max-liklihood fit before running BIC comparisons"
    post = radvel.posterior.load(status.get('fit', 'postfile'))

    for btype in args.type:
        print "performing bic comparison: {}".format(btype)

        if btype == 'nplanets':
            statsdict = radvel.fitting.model_comp(post, verbose=False)
            savestate['nplanets'] = statsdict


    save_status(statfile, 'bic', savestate)

def tables(args):
    """Generate TeX code for tables in summary report

    Args:
        args (ArgumentParser): command line arguments
    """

    config_file = args.setupfn
    conf_base = os.path.basename(config_file).split('.')[0]
    statfile = os.path.join(args.outputdir,
                            "{}_radvel.stat".format(conf_base))
    status = load_status(statfile)

    assert status.getboolean('mcmc', 'run'), \
        "Must run MCMC before making tables"

    P, post = radvel.utils.initialize_posterior(config_file)
    post = radvel.posterior.load(status.get('fit', 'postfile'))
    chains = pd.read_csv(status.get('mcmc', 'chainfile'))
    report = radvel.report.RadvelReport(P, post, chains)

    for tabtype in args.type:
        print "Generating LaTeX code for {} table".format(tabtype)                

        if tabtype == 'params':
            tex = report.tabletex(tabtype=tabtype)

        if tabtype == 'priors':
            tex = report.tabletex(tabtype=tabtype)

        if tabtype == 'nplanets':
            assert status.has_option('bic', 'nplanets'), \
                "Must run BIC comparison before making comparison tables"

            compstats = eval(status.get('bic', 'nplanets'))
            report = radvel.report.RadvelReport(
                P, post, chains, compstats=compstats
            )
            tex = report.tabletex(tabtype='nplanets')

        saveto = os.path.join(
            args.outputdir, '{}_{}_.tex'.format(conf_base,tabtype)
        )
        with open(saveto, 'w') as f:
            print >>f, tex

        savestate = {'{}_tex'.format(tabtype): os.path.abspath(saveto)}
        save_status(statfile, 'table', savestate)

                

def derive(args):
    """Derive physical parameters from posterior samples

    Args:
        args (ArgumentParser): command line arguments
    """

    config_file = args.setupfn
    conf_base = os.path.basename(config_file).split('.')[0]
    statfile = os.path.join(args.outputdir,
                            "{}_radvel.stat".format(conf_base))
    status = load_status(statfile)

    msg = "Multiplying mcmc chains by physical parameters for {}".format(
        conf_base
    )
    print msg

    assert status.getboolean('mcmc', 'run'), \
        "Must run MCMC before making tables"

    P, post = radvel.utils.initialize_posterior(config_file)
    post = radvel.posterior.load(status.get('fit', 'postfile'))
    chains = pd.read_csv(status.get('mcmc', 'chainfile'))

    mstar = np.random.normal(
        loc=P.stellar['mstar'], scale=P.stellar['mstar_err'], 
        size=len(chains)
        )

    # Convert chains into CPS basis
    cpschains = chains.copy()
    for par in post.params.keys():
        if (par not in post.vary.keys()) or (not post.vary[par]):
            cpschains[par] = post.params[par]


    cpschains = post.params.basis.to_cps(cpschains)

    savestate = {'run': True}
    outcols = []
    for i in np.arange(1, P.nplanets +1, 1):
        # Grab parameters from the chain
        def _has_col(key):
            cols = list(cpschains.columns)
            return cols.count('{}{}'.format(key,i))==1

        def _get_param(key):
            if _has_col(key):
                return cpschains['{}{}'.format(key,i)]
            else:
                return P.params['{}{}'.format(key,i)]

        def _set_param(key, value):
            chains['{}{}'.format(key,i)] = value

        def _get_colname(key):
            return '{}{}'.format(key,i)

        per = _get_param('per')
        k = _get_param('k')
        e = _get_param('e')

        mpsini = radvel.orbit.Msini(k, per, mstar, e, Msini_units='earth')
        _set_param('mpsini',mpsini)

        outcols.append(_get_colname('mpsini'))

        try:
            rp = np.random.normal(
                loc=P.planet['rp{}'.format(i)], 
                scale=P.planet['rp_err{}'.format(i)],
                size=len(chains)
            )

            _set_param('rp',rp)
            _set_param('rhop', radvel.orbit.density(mpsini, rp))

            outcols.append(_get_colname('rhop'))
        except (AttributeError, KeyError):
            pass

    print "Derived parameters:", outcols

    csvfn = os.path.join(args.outputdir, conf_base+'_derived.csv.tar.bz2')
    chains.to_csv(csvfn, columns=outcols, compression='bz2')
    savestate['chainfile'] = os.path.abspath(csvfn)

    save_status(statfile, 'derive', savestate)


def report(args):
    """Generate summary report

    Args:
        args (ArgumentParser): command line arguments
    """

    
    config_file = args.setupfn
    conf_base = os.path.basename(config_file).split('.')[0]
    print "Assembling report for {}".format(conf_base)

    statfile = os.path.join(args.outputdir,
                            "{}_radvel.stat".format(conf_base))

    status = load_status(statfile)

    P, post = radvel.utils.initialize_posterior(config_file)
    post = radvel.posterior.load(status.get('fit', 'postfile'))
    chains = pd.read_csv(status.get('mcmc', 'chainfile'))

    try:
        compstats = eval(status.get('bic', args.comptype))
    except:
        print "WARNING: Could not find {} BIC model comparison\
in {}.\nPlease make sure that you have run `radvel bic -t {}` if you would\
like to include\nthe model comparison table in the report.".format(args.comptype,
                                                            statfile,
                                                            args.comptype)
        compstats = None

    report = radvel.report.RadvelReport(P, post, chains, compstats=compstats)
    report.runname = conf_base

    report_depfiles = []
    for ptype,pfile in status.items('plot'):
        report_depfiles.append(pfile)

    with radvel.utils.working_directory(args.outputdir):
        rfile = os.path.join(conf_base+"_results.pdf")
        report_depfiles = [os.path.basename(p) for p in report_depfiles]
        report.compile(
            rfile, depfiles=report_depfiles, latex_compiler=args.latex_compiler
        )

    
def save_status(statfile, section, statevars):
    """Save pipeline status

    Args:
        statfile (string): name of output file
        section (string): name of section to write
        statevars (dict): dictionary of all options to populate
           the specified section
    """

    config = ConfigParser.RawConfigParser()
    
    if os.path.isfile(statfile):
        config.read(statfile)

    if not config.has_section(section):
        config.add_section(section)
        
    for key,val in statevars.items():
        config.set(section, key, val)

    with open(statfile, 'w') as f:
        config.write(f)

def load_status(statfile):
    """Load pipeline status

    Args:
        statfile (string): name of ConfigParser file

    Returns:
        ConfigParser.RawConfigParser
    """
    
    config = ConfigParser.RawConfigParser()
    gl = config.read(statfile)

    return config



import imp
import os
from decimal import Decimal
from contextlib import contextmanager
import warnings

import numpy as np
from datetime import datetime, timedelta

import radvel

def initialize_posterior(config_file):

    system_name = os.path.basename(config_file).split('.')[0]
    P = imp.load_source(system_name, os.path.abspath(config_file))
    system_name = P.starname

    cpsparams = P.params.basis.to_cps(P.params)
    params = P.params.basis.from_cps(cpsparams, P.fitting_basis, keep=False)
    
    for key in params.keys():
        if key.startswith('logjit'):
            msg = """
Fitting log(jitter) is depreciated. Please convert your config
files to initialize 'jit' instead of 'logjit' parameters.
Converting 'logjit' to 'jit' for you now.
"""
            warnings.warn(msg, DeprecationWarning, stacklevel=2)
            newkey = key.replace('logjit', 'jit')
            params[newkey] = np.exp(params[key])
            P.vary[newkey] = P.vary[key]
            del P.vary[key]
            del params[key]

    iparams = params.copy()

    # Make sure we don't have duplicate indicies in the DataFrame
    P.data = P.data.reset_index(drop=True)
    
    # initialize RVmodel object
    mod = radvel.RVModel(params, time_base=P.time_base)   

    # initialize RVlikelihood objects for each instrument
    telgrps = P.data.groupby('tel').groups
    likes = {}
    for inst in P.instnames:
        likes[inst] = radvel.likelihood.RVLikelihood(
            mod, P.data.iloc[telgrps[inst]].time,
            P.data.iloc[telgrps[inst]].mnvel,
            P.data.iloc[telgrps[inst]].errvel, suffix='_'+inst
        )
        likes[inst].params['gamma_'+inst] = iparams['gamma_'+inst]
        likes[inst].params['jit_'+inst] = iparams['jit_'+inst]

    like = radvel.likelihood.CompositeLikelihood(likes.values())

    # Set fixed/vary parameters
    like.vary.update(P.vary)
    
    # Initialize Posterior object
    post = radvel.posterior.Posterior(like)
    post.priors = P.priors
    return P, post


def round_sig(x, sig=2):
    """
    Round to the requested number of significant figures.

    Args:
        x (float): value
        sig (int): (optional) desired number of significant figures

    Returns:
        float: `x` rounded to `sig` significant figures
    """

    if x == 0 or np.isnan(x): return 0.0
    return round(x, sig-int(floor(log10(abs(x))))-1)

def sigfig(med, errlow, errhigh=None):
    """
    Format values with errors into an equal number of signficant figures.

    Args:
        med (float): median value
        errlow (float): lower errorbar
        errhigh (float): upper errorbar

    Returns:
        tuple: (med,errlow,errhigh) rounded to the lowest number of significant figures

    """
    
    if errhigh==None: errhigh = errlow
        
    ndec = Decimal(str(errlow)).as_tuple().exponent
    if abs(Decimal(str(errhigh)).as_tuple().exponent) > abs(ndec): ndec = Decimal(str(errhigh)).as_tuple().exponent
    if ndec < -1:
            tmpmed = round(med,abs(ndec))
            p = 0
            while tmpmed == 0:
                tmpmed = round(med, abs(ndec)+p)
                p += 1
            med = tmpmed
    elif (ndec == -1 and str(errhigh)[-1] == '0') and (ndec == -1 and str(errlow)[-1] == '0') or ndec == 0:
            errlow = int(round_sig(errlow))
            errhigh = int(round(errhigh))
            med = int(round(med))
    else: med = round(med,abs(ndec))

    return med, errlow, errhigh

def time_print(tdiff):
    units = 'seconds'
    if tdiff > 60:
        tdiff /= 60
        units = 'minutes'
        if tdiff > 60:
            tdiff /= 60
            units = 'hours'
            if tdiff > 24:
                tdiff /= 24
                units = 'days'
    return tdiff, units

def timebin(time, meas, meas_err, binsize):
#  This routine bins a set of times, measurements, and measurement errors 
#  into time bins.  All inputs and outputs should be floats or double. 
#  binsize should have the same units as the time array.
#  - from Andrew Howard, ported to Python by BJ Fulton

    ind_order = np.argsort(time)
    time = time[ind_order]
    meas = meas[ind_order]
    meas_err = meas_err[ind_order]
    ct=0
    while ct < len(time):
        ind = np.where((time >= time[ct]) & (time < time[ct]+binsize))[0]
        num = len(ind)
        wt = (1./meas_err[ind])**2.     #weights based in errors
        wt = wt/np.sum(wt)              #normalized weights
        if ct == 0:
            time_out = [np.sum(wt*time[ind])]
	    meas_out = [np.sum(wt*meas[ind])]
	    meas_err_out = [1./np.sqrt(np.sum(1./(meas_err[ind])**2))]
        else:
            time_out.append(np.sum(wt*time[ind]))
	    meas_out.append(np.sum(wt*meas[ind]))
	    meas_err_out.append(1./np.sqrt(np.sum(1./(meas_err[ind])**2)))
        ct += num

    return time_out, meas_out, meas_err_out


def bintels(t, vel, err, telvec, binsize=1/2.):
    # Bin RV data with bins of with binsize in the units of t.
    # Will not bin data from different telescopes together since there may
    # be offsets between them.

    ntels = len(np.unique(telvec))
    if ntels == 1:
        t_bin, vel_bin, err_bin = timebin(t, vel, err, binsize=binsize)
        return t_bin, vel_bin, err_bin, telvec
    
    uniqorder = np.argsort(np.unique(telvec,return_index=1)[1])
    uniqsort = np.unique(telvec)[uniqorder]
    rvtimes = np.array([])
    rvdat = np.array([])
    rverr = np.array([])
    newtelvec = np.array([])
    for i,tel in enumerate(uniqsort):
        pos = np.where(telvec == tel)
        t_bin, vel_bin, err_bin = timebin(
            t[pos], vel[pos], err[pos], binsize=binsize
        )
        rvtimes = np.hstack((rvtimes, t_bin))
        rvdat = np.hstack((rvdat, vel_bin))
        rverr = np.hstack((rverr, err_bin))
        newtelvec = np.hstack((newtelvec, np.array([tel]*len(t_bin))))
        
    return rvtimes, rvdat, rverr, newtelvec

def fastbin(x,y,nbins=30):
    n, _ = np.histogram(x, bins=nbins)
    sy, _ = np.histogram(x, bins=nbins, weights=y)
    sy2, _ = np.histogram(x, bins=nbins, weights=y*y)
    bindat = sy / n
    binerr = np.sqrt(sy2/n - bindat*bindat) / np.sqrt(n)
    bint = (_[1:] + _[:-1])/2.

    binN = n
    pos = binN >= 3# 0.5 * np.mean(binN)
    bint = bint[pos]
    bindat = bindat[pos]
    binerr = binerr[pos]

    pos = bint>0
    bint = bint[pos]
    bindat = bindat[pos]
    binerr = binerr[pos]
    return bint,bindat,binerr

def round_sig(x, sig=2):
    if x == 0: return 0.0
    return round(x, sig-int(np.floor(np.log10(abs(x))))-1)

def t_to_phase(params, t, num_planet, cat=False):
    if ('tc%i' % num_planet) in params:
        timeparam = 'tc%i' % num_planet
    elif ('tp%i' % num_planet) in params:
        timeparam = 'tp%i' % num_planet
        
    P = params['per%i' % num_planet]
    tc = params[timeparam]
    phase = np.mod(t - tc, P) 
    phase /= P
    if cat: phase = np.concatenate((phase,phase+1))
    return phase

def phase_to_t(params, phase, num_planet):
    if ('tc%i' % num_planet) in params:
        timeparam = 'tc%i' % num_planet
    elif ('tp%i' % num_planet) in params:
        timeparam = 'tp%i' % num_planet
        
    P = params['per%i' % num_planet]
    tc = params[timeparam]
    t = phase * P
    t += tc
    return t

@contextmanager
def working_directory(dir):
    """Do something in a directory

    Function to use with `with` statements.

    Args:
       dir (string): name of directory to work in
    
    Example:
        >>> with workdir('/temp'):
            # do something within the /temp directory
    """
    cwd = os.getcwd()
    os.chdir(dir)
    try:
        yield
    finally:
        os.chdir(cwd)

def cmd_exists(cmd):
    return any(
        os.access(os.path.join(path, cmd), os.X_OK) 
        for path in os.environ["PATH"].split(os.pathsep))

def date2jd(date):
    """
    Convert datetime object to JD"

    Args:
        date (datetime.datetime): date to convert
    Returns:
        float: Julian date
     """
    
    jd_td = date - datetime(2000,1,1,12,0,0)
    jd = 2451545.0 + jd_td.days + jd_td.seconds/86400.0
    return jd

def jd2date(jd):
    """
    Convert JD to datetime.datetime object

    Args:
        jd (float): Julian date
    Returns:
        datetime.datetime: calendar date
    """
    
    mjd = jd - 2400000.5
    td = timedelta(days=mjd)
    dt = datetime(1858,11,17,0,0,0) + td

    return dt

def t2dt(atime):
    """
    Convert fractional year (a float) to datetime

    Args:
        atime (float): Decimal year
    Returns:
        datetime.datetime: calendar date

    """
    year = int(atime)
    remainder = atime - year
    boy = datetime(year, 1, 1)
    eoy = datetime(year + 1, 1, 1)
    seconds = remainder * (eoy - boy).total_seconds()
    return boy + timedelta(seconds=seconds)

def geterr(vec, angular=False):
    """
    Calculate median, 15.9, and 84.1 percentile values
    for a given vector."

    Args:
        vec (array): vector, usually an MCMC chain for one parameter
        angular (bool): (optional) Is this an angular parameter?
            if True vec should be in radians. This will perform
            some checks to ensure proper boundary wrapping.

    Returns:
        tuple: 50, 15.9 and 84.1 percentiles
    """

    if angular:
        val, edges = np.histogram(vec, bins=50)
        med = edges[np.argmax(val)]
        if med > np.radians(90):
            vec[vec<np.radians(0)] = vec[vec<np.radians(0)] + np.radians(360)
        if med <= np.radians(-90):
            vec[vec>=np.radians(0)] = vec[vec>=np.radians(0)] - np.radians(360)
        med = np.median(vec)
    else:
        med = np.median(vec)
        
    s = sorted(vec)
    errlow = med - s[int(0.159*len(s))]
    errhigh = s[int(0.841*len(s))] - med
            
    return med, errlow, errhigh



import numpy as np

class Likelihood(object):
    """
    Generic Likelihood
    """
    def __init__(self, model, x, y, yerr, extra_params=[]):
        self.model = model
        self.params = model.params

        self.x = np.array(x) # Variables must be arrays.
        self.y = np.array(y) # Pandas data structures lead to problems.
        self.yerr = np.array(yerr)
        self.params.update({}.fromkeys(extra_params, np.nan) ) 
        self.uparams = None
        
        vary = {}.fromkeys(self.params.keys(), True)
        self.vary = vary

    def __repr__(self):
        s = ""
        if self.uparams is None:
            s +=  "{:<20s}{:>15s}{:>10s}\n".format(
                'parameter', 'value', 'vary'
                )
            keys = self.params.keys()
            keys.sort()
            for key in keys:
                if key in self.vary.keys():
                    vstr = str(self.vary[key])
                else:
                    vstr = ""
                
                if (key.startswith('tc') or key.startswith('tp')) and self.params[key] > 1e6:
                    par = self.params[key] - 2450000
                else:
                    par = self.params[key]

                s +=  "{:20s}{:15g} {:>10s}\n".format(
                    key, par, vstr
                     )
        else:
            s = ""
            s +=  "{:<20s}{:>15s}{:>10s}{:>10s}\n".format(
                'parameter', 'value', '+/-', 'vary'
                )
            keys = self.params.keys()
            keys.sort()
            for key in keys:
                if key in self.vary.keys():
                    vstr = str(self.vary[key])
                else:
                    vstr = ""
                if key in self.uparams.keys():
                    err = self.uparams[key]
                else:
                    err = 0
                    
                if (key.startswith('tc') or key.startswith('tp')) and self.params[key] > 1e6:
                    par = self.params[key] - 2450000
                else:
                    par = self.params[key]
                    
                s +=  "{:20s}{:15g}{:10g}{:>10s}\n".format(
                    key, par, err, vstr
                     )

                
        return s

    def set_vary_params(self, params_array):
        i = 0
        for key in self.list_vary_params():
            # flip sign for negative jitter
            if key.startswith('jit') and params_array[i] < 0:
                params_array[i] = -params_array[i]
                    
            self.params[key] = params_array[i]
            i+=1

        assert i==len(params_array), \
            "length of array must match number of varied parameters"

    def get_vary_params(self):
        params_array = []
        for key in self.params.keys():
            if self.vary[key]:
                params_array += [ self.params[key] ]
                
        params_array = np.array(params_array)
        return params_array

    def list_vary_params(self):
        return [key for key in self.params if self.vary[key] ]

    def residuals(self):
        return self.y - self.model(self.x) 

    def neglogprob(self):
        return -1.0 * self.logprob()

    def neglogprob_array(self, params_array):
        return -self.logprob_array(params_array)

    def logprob_array(self, params_array):
        self.set_vary_params(params_array)
        _logprob = self.logprob()
        return _logprob

class CompositeLikelihood(Likelihood):
    def __init__(self, like_list):
        """Composite Likelihood

        A thin wrapper to combine multiple `Likelihood`
        objects. One `Likelihood` applies to a dataset from
        a particular instrument.

        Args:
            like_list (list): list of `radvel.likelihood.RVLikelihood` objects
        """
    
        
        self.nlike = len(like_list)

        like0 = like_list[0]
        params = like0.params
        self.model = like0.model
        self.x = like0.x
        self.y = like0.y - params[like0.gamma_param]
        #self.yerr = np.sqrt(like0.yerr**2 + like0.params[like0.jit_param]**2)
        self.yerr = like0.yerr
        self.telvec = like0.telvec
        self.extra_params = like0.extra_params
        self.suffixes = like0.suffix
        self.uparams = like0.uparams
        
        for i in range(1,self.nlike):
            like = like_list[i]
            
            self.x = np.append(self.x,like.x)
            self.y = np.append(self.y, like.y - like.params[like.gamma_param])
            #self.yerr = np.append(self.yerr, np.sqrt(like.yerr**2 + like.params[like.jit_param]**2))
            self.yerr = np.append(self.yerr, like.yerr)
            self.telvec = np.append(self.telvec, like.telvec)
            self.extra_params = np.append(self.extra_params, like.extra_params)
            self.suffixes = np.append(self.suffixes, like.suffix)
            try:
                self.uparams = self.uparams.update(like.uparams)
            except AttributeError:
                self.uparams = None
            
            assert like.model is like0.model, \
                "Likelihoods must use the same model"

            for k in like.params:
                if params.has_key(k):
                    assert like.params[k] is params[k]
                else:
                    params[k] = like.params[k]


        self.extra_params = list(set(self.extra_params))
        self.params = params
        self.vary = {}.fromkeys(params.keys(),True)
        self.like_list = like_list

    def logprob(self):
        """
        See `radvel.likelihood.RVLikelihood.logprob`
        """
        
        _logprob = 0
        for like in self.like_list:
            _logprob += like.logprob()
        return _logprob

    def residuals(self):
        """
        See `radvel.likelihood.RVLikelihood.residuals`
        """

        res = self.like_list[0].residuals()
        for like in self.like_list[1:]:
            res = np.append(res,like.residuals())

        return res

    def errorbars(self):
        """
        See `radvel.likelihood.RVLikelihood.errorbars`
        """
        err = self.like_list[0].errorbars()
        for like in self.like_list[1:]:
            err = np.append(err,like.errorbars())

        return err

        

class RVLikelihood(Likelihood):
    """RV Likelihood

    The Likelihood object for a radial velocity dataset

    Args:
        model (radvel.model.RVModel): RV model object
        t (array): time array
        vel (array): array of velocities
        errvel (array): array of velocity uncertainties
        suffix (string): suffix to identify this Likelihood object
           useful when constructing a `CompositeLikelihood` object.

    """
    
    def __init__(self, model, t, vel, errvel, suffix=''):
        self.gamma_param = 'gamma'+suffix
        self.jit_param = 'jit'+suffix

        if suffix.startswith('_'): self.suffix = suffix[1:]
        else: self.suffix = suffix

        self.telvec = np.array([self.suffix]*len(t))
        
        self.extra_params = [self.gamma_param, self.jit_param]
        super(RVLikelihood, self).__init__(
            model, t, vel, errvel, extra_params=self.extra_params
            )

    def residuals(self):
        """Residuals

        Data minus model
        """
        
        return self.y - self.params[self.gamma_param] - self.model(self.x)

    def errorbars(self):
        """
        Return uncertainties with jitter added
        in quadrature.

        Returns:
            array: uncertainties
        
        """
        return np.sqrt(self.yerr**2 + self.params[self.jit_param]**2)

    def logprob(self):
        """
        Return log-likelihood given the data and model.
        Priors are not applied here.

        Returns:
            float: Natural log of likelihood
        """
        
        sigma_jit = self.params[self.jit_param]
        residuals = self.residuals()
        loglike = loglike_jitter(residuals, self.yerr, sigma_jit)
        
        return loglike

def loglike_jitter(residuals, sigma, sigma_jit):
    """
    Log-likelihood incorporating jitter

    See equation (1) in Howard et al. 2014. Returns loglikelihood, where 
    sigma**2 is replaced by sigma**2 + sigma_jit**2. It penalizes
    excessively large values of jitter
    
    Args:
        residuals (array): array of residuals
        sigma (array): array of measurement errors
        sigma_jit (float): jitter

    Returns:
        float: log-likelihood
    """
    sum_sig_quad = sigma**2 + sigma_jit**2
    penalty = np.sum( np.log( np.sqrt( 2 * np.pi * sum_sig_quad ) ) )
    chi2 = np.sum(residuals**2 / sum_sig_quad)
    loglike = -0.5 * chi2 - penalty 
    return loglike





import numpy as np

# Try to import Kepler's equation solver written in C
try:
    from . import _kepler 
    cext = True
except ImportError:
    print "WARNING: KEPLER: Unable to import C-based Kepler's\
equation solver. Falling back to the slower NumPy implementation."
    cext = False

def rv_drive(t, orbel, use_C_kepler_solver=cext):
    """RV Drive
    
    Args:
        t (array of floats): times of observations
        orbel (array of floats): [per, tp, e, om, K].\
            Omega is expected to be\
            in radians
        use_C_kepler_solver (bool): (default: True) If \
            True use the Kepler solver written in C, else \
            use the Python/NumPy version.

    Returns:
        rv: (array of floats): radial velocity model
    
    """
    
    # unpack array
    per, tp, e, om, k = orbel
    
    # Error checking
    if e == 0.0:
        M = 2 * np.pi * ( ((t - tp) / per) - np.floor( (t - tp) / per ) )
        return k * np.cos( M + om )
    
    if per < 0: per = 1e-4
    if e < 0: e = 0
    if e > 0.99: e = 0.99


    # Calculate the approximate eccentric anomaly, E1, via the mean anomaly  M.
    if use_C_kepler_solver:
        rv = _kepler.rv_drive_array(t, per, tp, e, om, k)
    else:
        M = 2 * np.pi * ( ((t - tp) / per) - np.floor( (t - tp) / per ) )
        eccarr = np.zeros(t.size) + e
        E1 = kepler(M, eccarr)
        # Calculate nu
        nu = 2 * np.arctan( ( (1+e) / (1-e) )**0.5 * np.tan( E1 / 2 ) )
        # Calculate the radial velocity
        rv = k * ( np.cos( nu + om ) + e * np.cos( om ) ) 
    
    return rv

def kepler(inbigM, inecc):
    """Solve Kepler's Equation

    Args:
        inbigM (array): input Mean annomaly
        inecc (array): eccentricity

    Returns:
        eccentric annomaly: array
    
    """
    
    Marr = inbigM  # protect inputs; necessary?
    eccarr = inecc
    conv = 1.0e-12  # convergence criterion
    k = 0.85

    Earr = Marr + np.sign(np.sin(Marr)) * k * eccarr  # first guess at E
    # fiarr should go to zero when converges
    fiarr = ( Earr - eccarr * np.sin(Earr) - Marr)  
    convd = np.abs(fiarr) > conv  # which indices have not converged
    nd = np.sum(convd == True) # number of converged elements
    count = 0

    while nd > 0:  # while unconverged elements exist
        count += 1
        
        M = Marr[convd]  # just the unconverged elements ...
        ecc = eccarr[convd]
        E = Earr[convd]

        fi = fiarr[convd]  # fi = E - e*np.sin(E)-M    ; should go to 0
        fip = 1 - ecc * np.cos(E) # d/dE(fi) ;i.e.,  fi^(prime)
        fipp = ecc * np.sin(E) # d/dE(d/dE(fi)) ;i.e.,  fi^(\prime\prime)
        fippp = 1 - fip  # d/dE(d/dE(d/dE(fi))) ;i.e.,  fi^(\prime\prime\prime)

        # first, second, and third order corrections to E
        d1 = -fi / fip 
        d2 = -fi / (fip + d1 * fipp / 2.0)
        d3 = -fi / (fip + d2 * fipp/ 2.0 + d2 * d2 * fippp / 6.0) 
        E = E + d3
        Earr[convd] = E
        fiarr = ( Earr - eccarr * np.sin( Earr ) - Marr) # how well did we do?
        convd = np.abs(fiarr) > conv  #test for convergence
        nd = np.sum(convd == True)
        
    if Earr.size > 1: 
        return Earr
    else: 
        return Earr[0]

def profile():
    # Profile and compare C-based Kepler solver with
    # Python/Numpy implementation

    import timeit
    
    ecc = 0.20
    orbel = [32.468, 2456000, ecc, np.pi/2, 10.0]
    numloops = 10000

    
    for size in [10,30,100,300,1000,3000]:

        setup = """\
from radvel.kepler import rv_drive
import numpy as np

gc.enable()

ecc = %f
orbel = [32.468, 2456000, ecc, np.pi/2, 10.0]

t = np.linspace(2455000, 2457000, %d)
""" % (ecc, size)
        
        
        print "\nProfiling pure C code for an RV time series with {} observations".format(size)
        tc = timeit.timeit('rv_drive(t, orbel, use_C_kepler_solver=True)', setup=setup, number=numloops)
        print "Ran %d model calculations in %5.3f seconds" % (numloops, tc)

        print "Profiling Python code for an RV time series with {} observations".format(size)
        tp = timeit.timeit('rv_drive(t, orbel, use_C_kepler_solver=False)', setup=setup, number=numloops)
        print "Ran %d model calculations in %5.3f seconds" % (numloops, tp)
        print "The C version runs %5.2f times faster" % (tp/tc)



"""
Command Line Interface
"""
import os
from argparse import ArgumentParser
import imp
import warnings
import pickle

import radvel.driver

warnings.filterwarnings("ignore")
warnings.simplefilter('once', DeprecationWarning)

def main():
    psr = ArgumentParser(
        description="RadVel: The Radial Velocity Toolkit", prog='radvel'
    )

    subpsr = psr.add_subparsers(title="subcommands", dest='subcommand')

    # In the parent parser, we define arguments and options common to
    # all subcommands.
    psr_parent = ArgumentParser(add_help=False)
    psr_parent.add_argument(
        '-d', dest='outputdir', type=str,
        help="Working directory. Default is the same as the \
        configuration file (without .py)"
    )
    psr_parent.add_argument('-s','--setup',
        dest='setupfn', type=str, 
        help="Setup file."
    )

    # Fitting    
    psr_fit = subpsr.add_parser(
        'fit', parents=[psr_parent],
        description="Perform max-likelihood fitting"
    )
    psr_fit.set_defaults(func=radvel.driver.fit)

    
    # Plotting
    psr_plot = subpsr.add_parser('plot', parents=[psr_parent],)
    psr_plot.add_argument('-t','--type',
        type=str, nargs='+',
        choices=['rv','corner','trend', 'derived'],
        help="type of plot(s) to generate"
    )
    psr_plot.add_argument(
        '--plotkw', dest='plotkw',action='store', default="{}", type=eval,
        help='''
        Dictionary of keywords sent to rv_multipanel_plot. 
        E.g. --plotkw "{'yscale_auto': True}"'
        ''',
    )
    
    psr_plot.set_defaults(func=radvel.driver.plots)

    
    # MCMC
    psr_mcmc = subpsr.add_parser(
        'mcmc', parents=[psr_parent],
        description="Perform MCMC exploration"
    )
    psr_mcmc.add_argument(
        '--nsteps', dest='nsteps', action='store',default=20000, type=float, 
        help='Number of steps per chain [20000]',)
    psr_mcmc.add_argument(
        '--nwalkers', dest='nwalkers', action='store', default=50, type=int,
        help='Number of walkers. [50]', 
    )
    psr_mcmc.set_defaults(func=radvel.driver.mcmc)


    # Derive hysical parameters
    psr_physical = subpsr.add_parser(
        'derive', parents=[psr_parent],
        description="Multiply MCMC chains by physical parameters. MCMC must"
        + "be run first"
    )

    psr_physical.set_defaults(func=radvel.driver.derive)
    
    # BIC 
    psr_bic = subpsr.add_parser('bic', parents=[psr_parent],)
    psr_bic.add_argument('-t',
        '--type', type=str, nargs='+', 
        choices=['nplanets'],
        help="type of BIC comparison to perform"
    )
    psr_bic.set_defaults(func=radvel.driver.bic)

    # Tables
    psr_table = subpsr.add_parser('table', parents=[psr_parent],)
    psr_table.add_argument('-t','--type',
        type=str, nargs='+',
        choices=['params','priors', 'nplanets'],
        help="type of plot(s) to generate"
    )
    psr_table.add_argument(
        '--header', action='store_true',
        help="included latex column header. Default just prints data rows"
    )
    
    psr_table.set_defaults(func=radvel.driver.tables)

    
    # Report
    psr_report = subpsr.add_parser(
        'report', parents=[psr_parent], 
        description="Merge output tables and plots into LaTeX report"
    )
    psr_report.add_argument(
        '--comptype', dest='comptype', action='store',
        default='nplanets', type=str, 
        help='Type of BIC model comparison table to include. \
        Default: nplanets')

    psr_report.add_argument(
        '--latex-compiler', default='pdflatex', type=str, 
        help='Path to latex compiler'
        )

    psr_report.set_defaults(func=radvel.driver.report)
    
    args = psr.parse_args()

    if args.outputdir is None:
        setupfile = args.setupfn
        print setupfile
        system_name = os.path.basename(setupfile).split('.')[0]
        outdir = os.path.join('./', system_name)
        args.outputdir = outdir
            
    if not os.path.isdir(args.outputdir):
        os.mkdir(outdir)
        
    args.func(args)


if __name__ == '__main__':
    main()



#!/usr/bin/env python

def restrict(input, inmin, inmax, lims=None):
    import numpy as np
    
    min = inmin
    max = inmax
    n = input.size
    #inf = np.where(ravel(logical_not(finite(in))))[0]
    if lims == None:
        lims = 2
    elif lims == 3:   
        lims = 2
    else:
        nl = lims.size
    data = np.array(input,dtype=float)
    if n == 1:
        data = (input)*np.ones(2)
    range = max - min
    sdata = (data - min) / range
    w = np.where((data >= max) | (data < min))[0]
    nw = w.size
    while nw > 0:
        w = np.where((data >= max) | (data < min))[0]
        nw = w.size
        #if nw > 0: data[w] = data[w] + (np.floor(np.abs(data - min) / range) > 1) * range * (2 * (data[w] < max) - 1)
        data[w] = data[w] + (np.max(np.floor(np.abs(data - min) / range))) * range * (2 * (data[w] < max) - 1)
        #if nw - ninf == 0:  break
   
    return ((n == 1) and [data[0]] or [data])[0]

if __name__ == '__main__':
    import numpy as np
    
    x = np.arange(100,dtype=float) - 40
    y = restrict(x,20,80)
    print x
    print y



import numpy as np
from scipy.optimize import  brentq
from kepler import kepler
from astropy.time import Time
from astropy import constants as c
from astropy import units as u


# Normalization. 
#RV m/s of a 1.0 Jupiter mass planet tugging on a 1.0
# solar mass star on a 1.0 year orbital period
K_0 = 28.4329


def timetrans_to_timeperi(tc, per, ecc, omega):
    """
    Convert Time of Transit to Time of Periastron Passage

    Args:
        tc (float): time of transit    
        per (float): period [days]
        ecc (float): eccecntricity
        omega (float): longitude of periastron (radians)
    
    Returns:
        float: time of periastron passage

    """
    try:
        if ecc >= 1: return tc
    except ValueError:
        pass
    
    f = np.pi/2   - omega
    EE = 2 * np.arctan( np.tan(f/2) * np.sqrt((1-ecc)/(1+ecc)) )  # eccentric anomaly
    tp = tc - per/(2*np.pi) * (EE - ecc*np.sin(EE))      # time of periastron
    
    return tp
    

def timeperi_to_timetrans(tp, per, ecc, omega, secondary=0):
    """
    Convert Time of Periastron to Time of Transit

    Args:
        tp (float): time of periastron
        per (float): period [days]
        ecc (float): eccentricity
        omega (float): argument of peri (radians)

    Returns:
        float: time of inferior conjuntion (time of transit if system is transiting)
    
    """
    try:
        if ecc >= 1: return tp
    except ValueError:
        pass
    
    if secondary:
        f = 3*np.pi/2 - omega                      # true anomaly during secondary eclipse
    else:
        f = np.pi/2   - omega                      # true anomaly during transit

    EE = 2 * np.arctan( np.tan(f/2) * np.sqrt((1-ecc)/(1+ecc)) )  # eccentric anomaly
    tc = tp + per/(2*np.pi) * (EE - ecc*np.sin(EE))         # time of conjunction

    return tc


def true_anomaly(t, P, e):
    """
    Calculate the true annomoly for a given time, period, eccentricity.

    :param t: time (BJD_TDB)
    :type t: float

    :param P: period [days]
    :type P: float

    :param e: eccentricity
    :type e: float


    :return: True Annomoly
    
    """

    
    # f in Murray and Dermott p. 27
    tp = 0
    t = np.array([t])
    m = 2. * np.pi * (((t - tp) / P) - np.floor((t - tp) / P))
    e1 = kepler(m, np.array([e]))
    n1 = 1.0 + e
    n2 = 1.0 - e
    nu = 2.0 * np.arctan((n1 / n2)**0.5 * np.tan(e1 / 2.e0))
    if nu < 0:
        nu+=2*np.pi
    return nu



def next_epoch(P, epoch, now, nnext=10):
    """
    Given a reference epoch and a period, compute the next times this period and epoch will occur
    """
    
    phase_now = np.mod((now - epoch) / P,1)
    i = np.arange(1,nnext+1)
    time_from_now = (i - phase_now) * P
    time = now + time_from_now
    return time


def sqrtecosom_sqrtesinom_to_e_om(sqrtecosom,sqrtesinom):
    e = sqrtecosom**2 + sqrtesinom**2
    om = np.arctan2(sqrtesinom,sqrtecosom) * 360 / 2 / np.pi
    om = np.mod(om + 360, 360) # omega is positive
    return e,om

def e_om_to_ecosom_esinom(e,om):
    dtorad = 2 * np.pi / 360.
    ecosom = e * np.cos( dtorad*om )
    esinom = e * np.sin( dtorad*om )
    return ecosom, esinom

def semi_amplitude(Msini, P, Mtotal, e, Msini_units='jupiter'):
    """
    Compute Doppler semi-amplitude

    :param Msini: mass of planet [Mjup]
    :type Msini: float
    
    :param P: Orbital period [days]
    :type P: float
    
    :param Mtotal: Mass of star + mass of planet [Msun]
    :type Mtotal: float
    
    :param e: eccentricity
    :type e: float

    :param Msini_units: Units of returned Msini. Must be 'earth', or 'jupiter' (default 'jupiter').
    :type Msini_units: string

    :return: Doppler semi-amplitude [m/s]
    """
    if Msini_units.lower() == 'jupiter':
        K = K_0 * ( 1 - e**2 )**-0.5 * Msini * ( P / 365.0 )**-0.333 * \
            Mtotal**-0.667
    elif Msini_units.lower() == 'earth':
        K = K_0 * ( 1 - e**2 )**-0.5 * Msini * ( P / 365.0 )**-0.333 * \
            Mtotal**-0.667*(c.M_earth/c.M_jup).value
    else: 
        raise Exception("Msini_units must be 'earth', or 'jupiter'")
        
    return K


def Msini(K, P, Mtotal, e, Msini_units='earth'):
    """Calculate Msini

    Calculate Msini for a given K, P, stellar mass, and e
    
    Args:
        K (float): Doppler semi-amplitude [m/s]
        P (float): Orbital period [days]
        Mtotal (float): Mass of star + mass of planet [Msun]
        e (float): eccentricity
        Msini_units = (optional) Units of returned Msini. Must be 'earth', or 'jupiter' (default 'earth'). 
    Returns:
        float: Msini [units = Msini_units]
    
    """

    if Msini_units.lower() == 'jupiter':
        Msini = K / K_0 * np.sqrt(1.0 - e**2.0) * Mtotal**0.667 * \
            (P/365.0)**0.333
    elif Msini_units.lower() == 'earth':
        Msini = K / K_0 * np.sqrt(1.0 - e**2.0) * Mtotal**0.667 * \
            (P/365.0)**0.333*(c.M_jup/c.M_earth).value
    else: 
        raise Exception("Msini_units must be 'earth', or 'jupiter'")
    
    return Msini


def density(mass,radius, MR_units='earth'):
    """
    :param mass: mass, units = MR_units 
    :type mass: float

    :param radius: radius, units = MR_units 
    :type radius: float

    :param MR_units: (optional) units of mass and radius. Must be 'earth', or 'jupiter' (default 'earth').

    :return: density (g/cc)
    """
    mass = np.array(mass)
    radius = np.array(radius)
    if MR_units.lower() == 'earth':
        vol = 4./3.*np.pi * (radius * c.R_earth)**3
        rho = ((mass * c.M_earth / vol).to(u.g / u.cm**3)).value
    elif MR_units.lower() == 'jupiter':
        vol = 4./3.*np.pi * (radius * c.R_jup)**3
        rho = ((mass * c.M_jup / vol).to(u.g / u.cm**3)).value
    else: 
        raise Exception("MR_units must be 'earth', or 'jupiter'")
    return rho


def Lstar(Rstar,Teff):
    """
    :param Rstar: Radius (solar units)
    :type Rstar: float
    
    :param Teff: Teff (K)
    :type Teff: float
    

    :return: Luminosity (solar units)
    """
    return (Rstar)**2*(Teff/5770)**4


def Sinc(Lstar,A):
    """
    :param Lstar: Luminosity (solar-units)
    :type Lstar: float
    
    :param A: Semi-major axis (AU)
    :type A: float


    :return: Insolation (Earth-units)
    """
    Sinc = Lstar / A**2
    return Sinc


def Teq(Sinc):
    """
    :param Sinc: Insolation (Earth-units)
    :type Sinc: float

    :return: Equilibrium temperature
    """
    Sinc = np.array(Sinc)
    Se = 1300*u.W*u.m**-2
    Teq = (Sinc*Se / 4 / c.sigma_sb)**0.25 # Kelvin
    Teq = Teq.to(u.K).value
    return Teq
         



import emcee
import pandas as pd
import numpy as np
from scipy import optimize
import sys
import time
from radvel import utils

# Maximum G-R statistic to stop burn-in period
burnGR = 1.10

# Maximum G-R statistic for chains to be deemed well-mixed
maxGR = 1.03

def mcmc(likelihood, nwalkers=50, nrun=10000, threads=1, checkinterval=50):
    """Run MCMC

    Run MCMC chains using the emcee EnsambleSampler

    Args:
        likelihood (radvel.likelihood): radvel likelihood object
        nwalkers (int): number of MCMC walkers
        nrun (int): number of steps to take
        threads (int): number of CPU threads to utilize
        checkinterval (int): check MCMC convergence statistics every 
            `checkinterval` steps

    Returns:
        DataFrame: DataFrame containing the MCMC samples

    """
    nrun = int(nrun)
        
    # Get an initial array value
    p0 = likelihood.get_vary_params()
    ndim = p0.size
    p0 = np.vstack([p0]*nwalkers)
    p0 += [np.random.rand(ndim)*1e-5 for i in range(nwalkers)]
    sampler = emcee.EnsembleSampler( 
        nwalkers, ndim, likelihood.logprob_array, threads=threads
    )

    pos = p0    
    num_run = int(np.round(nrun / checkinterval))
    totsteps = nrun*nwalkers
    mixcount = 0
    burn_complete = False
    nburn = 0
    t0 = time.time()
    for r in range(num_run):
        t1 = time.time()
        pos, prob, state = sampler.run_mcmc(pos, checkinterval)
        t2 = time.time()

        rate = (checkinterval*nwalkers) / (t2-t1)
        ncomplete = sampler.flatlnprobability.shape[0] + nburn
        pcomplete = ncomplete/float(totsteps) * 100
        ar = sampler.acceptance_fraction.mean() * 100.

        tchains = sampler.chain.transpose()
        
        (ismixed, gr, tz) = gelman_rubin(tchains)
        mintz = min(tz)
        maxgr = max(gr)
        if ismixed: mixcount += 1
        else: mixcount = 0

        # Burn-in complete after maximum G-R statistic first reaches 1.10
        # reset sampler
        if not burn_complete and maxgr <= burnGR:
            sampler.reset()
            msg = (
                "\nDiscarding burn-in now that the chains are marginally "
                "well-mixed\n"
            )
            print msg
            nburn = ncomplete
            burn_complete = True

        if mixcount >= 5:
            tf = time.time()
            tdiff = tf - t0
            tdiff,units = utils.time_print(tdiff)
            msg = (
                "\nChains are well-mixed after {:d} steps! MCMC completed in "
                "{:3.1f} {:s}"
            ).format(ncomplete, tdiff, units)
            print msg
            break
        else:
            msg = (
                "{:d}/{:d} ({:3.1f}%) steps complete; "
                "Running {:.2f} steps/s; Mean acceptance rate = {:3.1f}%; "
                "Min Tz = {:.1f}; Max G-R = {:4.2f} \r"
            ).format(ncomplete, totsteps, pcomplete, rate, ar, mintz, maxgr)
            
            sys.stdout.write(msg)
            sys.stdout.flush()
            
    print "\n"        
    if ismixed and mixcount < 5: 
        msg = (
            "MCMC: WARNING: chains did not pass 5 consecutive convergence "
            "tests. They may be marginally well=mixed."
        )
        print msg
    elif not ismixed: 
        msg = (
            "MCMC: WARNING: chains did not pass convergence tests. They are "
            "likely not well-mixed."
        )
        print msg
    
    df = pd.DataFrame(
        sampler.flatchain,columns=likelihood.list_vary_params()
        )
    df['lnprobability'] = sampler.flatlnprobability
    return df

def draw_models_from_chain(mod, chain, t, nsamples=50):
    """Draw Models from Chain
    
    Given an MCMC chain of parameters, draw representative parameters
    and synthesize models.

    Args:
        mod (radvel.RVmodel) : RV model
        chain (DataFrame): pandas DataFrame with different values from MCMC 
            chain
        t (array): time range over which to synthesize models
        nsamples (int): number of draws
    
    Returns:
        array: 2D array with the different models as different rows

    """

    np.random.seed(0)
    chain_samples = chain.ix[np.random.choice(chain.index, nsamples)]
    models = []
    for i in chain_samples.index:
        params = np.array( chain.ix[i, mod.vary_parameters] )
        params = mod.array_to_params(params)
        models += [mod.model(params, t)]
    models = np.vstack(models)
    return models


def gelman_rubin(pars0, minTz=1000, maxGR=maxGR):
    """Gelman-Rubin Statistic

    Calculates the Gelman-Rubin statistic and the number of
    independent draws for each parameter, as defined by Ford et
    al. (2006) (http://adsabs.harvard.edu/abs/2006ApJ...642..505F).
    The chain is considered well-mixed if all parameters have a
    Gelman-Rubin statistic of <= 1.03 and >= 1000 independent draws.

    History: 
        2010/03/01 - Written: Jason Eastman - The Ohio State University        
        2012/10/08 - Ported to Python by BJ Fulton - University of Hawaii, 
            Institute for Astronomy
        2016/04/20 - Adapted for use in radvel. Removed "angular" parameter.

    Args:
        pars0 (array): A 3 dimensional array (NPARS,NSTEPS,NCHAINS) of
            parameter values
        minTz (int): (optional) minimum Tz to consider well-mixed
        maxGR (float): (optional) maximum Gelman-Rubin statistic to
            consider well-mixed

    Returns:
        (tuple): tuple containing:

            ismixed (bool): Are the chains well-mixed?
            gelmanrubin (array): An NPARS element array containing the
                Gelman-Rubin statistic for each parameter (equation
                25)
            Tz (array): An NPARS element array containing the number
                of independent draws for each parameter (equation 26)
    """


    pars = pars0.copy() # don't modify input parameters
    
    sz = pars.shape
    msg = 'MCMC: GELMAN_RUBIN: ERROR: pars must have 3 dimensions'
    assert pars.ndim == 3, msg 

    npars = float(sz[0])
    nsteps = float(sz[1])
    nchains = float(sz[2])

    msg = 'MCMC: GELMAN_RUBIN: ERROR: NSTEPS must be greater than 1'
    assert nsteps > 1, msg

    # Equation 21: W(z) in Ford 2006
    variances = np.var(pars,axis=1, dtype=np.float64)
    meanofvariances = np.mean(variances,axis=1)
    withinChainVariances = np.mean(variances, axis=1)
    
    # Equation 23: B(z) in Ford 2006
    means = np.mean(pars,axis=1)
    betweenChainVariances = np.var(means,axis=1, dtype=np.float64) * nsteps
    varianceofmeans = np.var(means,axis=1, dtype=np.float64) / (nchains-1)
    varEstimate = (
        (1.0 - 1.0/nsteps) * withinChainVariances 
        + 1.0 / nsteps * betweenChainVariances
    )
    
    bz = varianceofmeans * nsteps

    # Equation 24: varhat+(z) in Ford 2006
    varz = (nsteps-1.0)/bz + varianceofmeans

    # Equation 25: Rhat(z) in Ford 2006
    gelmanrubin = np.sqrt(varEstimate/withinChainVariances)

    # Equation 26: T(z) in Ford 2006
    vbz = varEstimate / bz
    tz = nchains*nsteps*vbz[vbz < 1]
    if tz.size == 0:
        tz = [-1]

    # well-mixed criteria
    ismixed = min(tz) > minTz and max(gelmanrubin) < maxGR
        
    return (ismixed, gelmanrubin, tz)



from .likelihood  import Likelihood
import copy
import time
import numpy as np
import pickle

class Posterior(Likelihood):
    """Posterior object

    Posterior object to be sent to the fitting routines.
    It is essentially the same as the Liklihood object,
    but priors are applied here.

    Args:
        likelihood (radvel.likelihood.Likelihood): Likelihood object
        params (radvel.model.RVParameters): parameters object

    Note:
        Append `radvel.prior.Prior` objects to the Posterior.priors list
        to apply priors in the likelihood calculations.
    """
    
    def __init__(self,likelihood, setup=None):
        self.likelihood = likelihood
        self.params = likelihood.params
        self.vary = likelihood.vary
        self.uparams = likelihood.uparams
        self.priors = []
    
    def __repr__(self):
        s = super(Posterior, self).__repr__()
        s += "\nPriors\n"
        s += "------\n"
        for prior in self.priors:
            s +=  prior.__repr__() + "\n"
        return s

    def logprob(self):
        """Log probability

        Log-probability for the likelihood given the list
        of priors in `Posterior.priors`.

        Returns:
            float: log probability of the likelihood + priors
        """
            
        _logprob = self.likelihood.logprob()
        for prior in self.priors:
            _logprob += prior( self.params )

        return _logprob

    def bic(self):
        """
        Calculate the Bayesian information criterion

        Returns:
            float: BIC
        """
        
        return -2.0 * self.logprob() + len(self.likelihood.get_vary_params()) + np.log(len(self.likelihood.y))

    
    def logprob_array(self, params_array):
        """Log probability for parameter vector

        Same as `self.logprob`, but will take a vector of
        parameter values. Useful as the objective function
        for routines that optimize a vector of parameter values
        instead of the dictionary-like format of the `radvel.model.RVParameters` object.

        Returns:
            float: log probability of the likelihood + priors

        """
        
        self.likelihood.set_vary_params(params_array)
        _logprob = self.logprob()
                
        return _logprob

    def writeto(self, filename):
        """
        Save posterior object to pickle file.

        Args:
            filename (string): full path to outputfile
        """
        
        with open(filename, 'wb') as f:
            pickle.dump(self, f)

def load(filename):
    """
    Load posterior object from pickle file.

    Args:
        filename (string): full path to pickle file
    """
        
    with open(filename, 'rb') as f:
        post = pickle.load(f)

    return post



from scipy import optimize
import numpy as np
import copy
import collections

def maxlike_fitting(post, verbose=True):
    """Maximum Likelihood Fitting

    Perform a maximum likelihood fit.

    Args:
        post (radvel.Posterior): Posterior object with initial guesses
        verbose (bool): (optional) Print messages and fitted values?

    Returns: radvel.Posterior : Posterior object with parameters
        updated their maximum likelihood values

    """
    
    post0 = copy.deepcopy(post)
    if verbose:
        print "Initial loglikelihood = %f" % post0.logprob()
        print "Performing maximum likelihood fit..."

    res  = optimize.minimize(
        post.neglogprob_array, post.get_vary_params(), method='Powell',
        options=dict(maxiter=100,maxfev=100000,xtol=1e-8)
    )

    cpspost = copy.deepcopy(post)
    cpsparams = post.params.basis.to_cps(post.params)
    cpspost.params.update(cpsparams)

    if verbose:
        print "Final loglikelihood = %f" % post.logprob()
        print "Best-fit parameters:"
        print cpspost
        
    return post
    

def model_comp(post, verbose=False):
    """Model Comparison

    Fit for planets adding one at a time.  Save results as list of
    posterior objects.

    Args:
        post (radvel.Posterior): posterior object for final best-fit solution 
            with all planets
        verbose (bool): (optional) print out statistics
        
    Returns:

        list of dictionaries : List of dictionaries with fit
            statistics. Each value in the dictionary is a tuple with
            the statistic value as the first element and a description
            of that statistic in the second element.
    """

    ipost = copy.deepcopy(post)
        
    num_planets = post.likelihood.model.num_planets

    statsdict = []
    
    for n in range(num_planets+1):
        pdict = collections.OrderedDict()
        if verbose:
            print "Testing %d planet model.\n\n" % n

        for par in post.params.keys():
            try:
                num = int(par[-1])
                if num > n:
                    if par.startswith('k') or par.startswith('logk'):
                        post.params[par] = 0.0
                    post.vary[par] = False
                else:
                    post.params[par] = ipost.params[par]
                    post.vary[par] = ipost.vary[par]
            except (ValueError, KeyError):
                pass

        post = maxlike_fitting(post, verbose=False)

        ndata = len(post.likelihood.y)
        nfree = len(post.get_vary_params())
        chi = np.sum((post.likelihood.residuals()/post.likelihood.yerr)**2)
        chi_red = chi / (ndata - nfree)

        if verbose:
            print post
            print "N_free = %d" % nfree
            print "RMS = %4.2f" % np.std(post.likelihood.residuals())
            print "logprob = %4.2f" % post.logprob()
            print "chi (no jitter) = %4.2f" % chi
            print "chi_red (no jitter) = %4.2f" % chi_red
            print "BIC = %4.2f" % post.bic()
        
        pdict['$N_{\\rm data}$'] = (ndata, 'number of measurements')
        pdict['$N_{\\rm free}$'] = (nfree, 'number of free parameters')
        pdict['RMS'] = (
            np.round(np.std(post.likelihood.residuals()), 2), 
            'RMS of residuals in m s$^{-1}$'
        )
        pdict['$\\chi^{2}$'] = (np.round(chi,2), "assuming no jitter")
        pdict['$\\chi^{2}_{\\nu}$'] = (
            np.round(chi_red,2), "assuming no jitter"
        )
        pdict['$\\ln{\\mathcal{L}}$'] = (
            np.round(post.logprob(),2), "natural log of the likelihood"
        )
        pdict['BIC'] = (
            np.round(post.bic(),2), 
            'Bayesian information criterion'
        )
        
        statsdict.append(pdict)

    return statsdict



import subprocess
import copy
import numpy as np
import os
import tempfile
import shutil

import radvel

print_basis = 'per tc e w k'
units = {'per': 'days',
         'tp': 'JD',
         'tc': 'JD',
         'e': '',
         'w': 'radians',
         'k': 'm s$^{-1}$',
         'logk': '$\\ln{(\\rm m\\ s^{-1})}$',
         'secosw': '',
         'sesinw': '',
         'gamma': 'm s$-1$',
         'jitter': 'm s$-1$',
         'logjit': '$\\ln{(\\rm m\\ s^{-1})}$',
         'jit': '$\\rm m\\ s^{-1}$',
         'dvdt': 'm s$^{-1}$ day$^{-1}$',
         'curv': 'm s$^{-1}$ day$^{-2}$'}


class RadvelReport():
    """Radvel report

    Class to handle the creation of the radvel summary PDF

    Args:
        planet (planet object): planet configuration object loaded in `kepfit.py` using `imp.load_source`
        post (radvel.posterior): radvel.posterior object containing the best-fit parameters in post.params
        chains (DataFrame): output DataFrame from a `radvel.mcmc` run
    """
    
    def __init__(self, planet, post, chains, compstats=None):
        self.planet = planet
        self.post = post
        
        self.starname = planet.starname
        self.starname_tex = planet.starname.replace('_', '\_')
        self.runname = self.starname_tex
                
        printpost = copy.deepcopy(post)
        printpost.params = printpost.params.basis.to_cps(printpost.params)
        printpost.params = printpost.params.basis.from_cps(printpost.params, print_basis)
        self.latex_dict = printpost.params.tex_labels()

        printchains = copy.deepcopy(chains)
        for p in post.params.keys():
            if p not in chains.columns:
                chains[p] = post.params[p]
        self.chains = printpost.params.basis.to_cps(chains)
        self.chains = printpost.params.basis.from_cps(chains, print_basis)
        self.quantiles = chains.quantile([0.159, 0.5, 0.841])

        self.compstats = compstats
        
    def _preamble(self):
        return """
\\documentclass{emulateapj}
\\usepackage{graphicx,textcomp}
\\begin{document}
\\shorttitle{Summary of \\texttt{RADVEL} results for %s}
""" % (self.starname_tex)

    def _postamble(self):
        return """
\\end{document}"""
    

    def texdoc(self):
        """TeX for entire document

        TeX code for the entire output results PDF

        Returns:
            string: TeX code for report
        """
        
        out = self._preamble() + self.tabletex()
        if os.path.exists(self.runname+"_rv_multipanel.pdf"):
            out += self.figtex(self.runname+"_rv_multipanel.pdf", caption=self._bestfit_caption())
        if os.path.exists(self.runname+"_corner.pdf"):
            out += self.figtex(self.runname+"_corner.pdf", caption="Posterior distributions for all free parameters.")
        if os.path.exists(self.runname+"_corner_derived_pars.pdf"):
            out += self.figtex(self.runname+"_corner_derived_pars.pdf", caption="Posterior distributions for all derived parameters.")

        out += self._postamble()
        
        return out

    def tabletex(self, tabtype='all'):
        return TexTable(self).tex(tabtype=tabtype)

    def figtex(self, infile, caption=""):
        """Generate TeX for figure

        Generate TeX to insert a figure into the report

        Args:
            infile (string): file name of figure
            caption (string): (optional) figure caption

        Returns:
            string: TeX code
        
        """
        fstr = """
\\begin{figure*}[!h]
\\centering
\\includegraphics[width=6.5in]{%s}
\\caption{%s}
\\end{figure*}
""" % (infile, caption)

        return fstr

    def _bestfit_caption(self):
        cap = """
Best-fit %d-planet Keplerian orbital model for %s.
The maximum likelihood model is plotted while the orbital parameters listed in Table \\ref{tab:params}
are the median values of the posterior distributions.
The thin blue line is the best fit %d-planet model. We add in quadrature the RV jitter term(s) listed in Table \\ref{tab:params}
with the measurement uncertainties for all RVs.
{\\bf b)} Residuals to the best fit %d-planet model.
{\\bf c)} RVs phase-folded to the ephemeris of planet %s. The Keplerian orbital models for all other planets (if any) have been subtracted.
The small point colors and symbols are the same as in panel {\\bf a}.
Red circles (if present) are the same velocities binned in 0.08 units of orbital phase.
The phase-folded model for planet %s is shown as the blue line.
""" % (self.post.params.num_planets, self.starname_tex, self.post.params.num_planets, self.post.params.num_planets, chr(int(1)+97), chr(int(1)+97))

        for i in range(1, self.post.params.num_planets):
            cap += "Panel {\\bf %s)} is the same as panel {\\bf %s)} but for planet %s %s.\n" % (chr(int(i)+99), chr(int(i)+98), self.starname_tex, chr(int(i)+98))

        return cap
              
    def compile(self, pdfname, latex_compiler='pdflatex', depfiles=[]):
        """Compile radvel report

        Compile the radvel report from a string containing TeX code
        and save the resulting PDF to a file.

        Args:
            pdfname (string): name of the output PDF file
            latex_compiler (string): path to latex
            depfiles (list): list of file names of dependencies needed for 
                LaTex compilation (e.g. figure files)
        """
        texname = os.path.basename(pdfname).split('.')[0] + '.tex'
        current = os.getcwd()
        temp = tempfile.mkdtemp()
        for fname in depfiles:
            shutil.copy2(os.path.join(current,fname), os.path.join(temp,fname))
        
        os.chdir(temp)

        f = open(texname, 'w')
        f.write(self.texdoc())
        f.close()
        try:
            for i in range(3):
                # LaTex likes to be compiled a few times
                # to get the table widths correct
                proc = subprocess.Popen(
                    [latex_compiler, texname], stdout=subprocess.PIPE, 
                )
                proc.communicate() # Let the subprocess complete
        except OSError:
            msg = """ 
WARNING: REPORT: could not run %s. Ensure that %s is in your PATH
or pass in the path as an argument
""" % (latex_compiler, latex_compiler)
            print msg
            return 

        shutil.copy(pdfname, current)
        shutil.copy(texname, current)
        
        shutil.rmtree(temp)
        os.chdir(current)

class TexTable(RadvelReport):
    """LaTeX table

    Class to handle generation of the LaTeX tables within the summary PDF.

    Args:
        report (radvel.report.RadvelReport): radvel report object
    """
    
    def __init__(self, report):
        self.report = report
        self.post = report.post
        self.quantiles = report.quantiles
        self.fitting_basis = report.post.params.basis.name
    
    def _header(self):
        fstr = """
\\begin{deluxetable}{lrrr}
\\tablecaption{MCMC Posteriors}
\\tablehead{\\colhead{Parameter} & \\colhead{Credible Interval} & \\colhead{Maximum Likelihood} & \\colhead{Units}}
\\startdata
"""
        return fstr

    def _footer(self):
        fstr = """
\\enddata
\\tablenotetext{}{%d links saved}
\\tablenotetext{}{Reference epoch for $\\gamma$,$\\dot{\\gamma}$,$\\ddot{\\gamma}$: %15.1f}
\\label{tab:params}
\\end{deluxetable}
""" % (len(self.report.chains),self.report.post.likelihood.model.time_base)
        return fstr
    
    def _row(self, param, unit):

        if unit == 'radians':
            med, low, high = radvel.utils.geterr(self.report.chains[param], angular=True)
        else:
            med = self.quantiles[param][0.5]
            low = self.quantiles[param][0.5] - self.quantiles[param][0.159]
            high = self.quantiles[param][0.841] - self.quantiles[param][0.5]

        maxlike = self.post.maxparams[param]
        
        tex = self.report.latex_dict[param]

        low = radvel.utils.round_sig(low)
        high = radvel.utils.round_sig(high)
        maxlike, errlow, errhigh = radvel.utils.sigfig(maxlike, low, high)
        med, errlow, errhigh = radvel.utils.sigfig(med, low, high)

        if errlow <= 1e-12 or errhigh <= 1e-12:
            med = maxlike = "$\\equiv$ %s" % round(self.quantiles[param][0.5],4)
            errfmt = ''
        else:
            if errhigh == errlow: errfmt = '$\pm %s$' % (errhigh)    
            else: errfmt = '$^{+%s}_{-%s}$' % (errhigh,errlow)

        row = "%s & %s %s & %s & %s\\\\\n" % (tex,med,errfmt,maxlike,unit)

        return row

    
    def _data(self, basis, sidehead=None, hline=False):

        tabledat = ""
        if hline:
            tabledat += "\\hline\n"
        if sidehead is not None:
            tabledat += "\\sidehead{%s}\n" % sidehead

        suffixes = ['_'+j for j in self.report.post.likelihood.suffixes]
            
        for n in range(1,self.report.planet.nplanets+1):
            for p in basis.split():
                
                unit = units.get(p, '')
                if unit == '':
                    for s in suffixes:
                        if s in p:
                            unit = units.get(p.replace(s, ''), '')
                            break
                        
                par = p+str(int(n))
                try:
                    tabledat += self._row(par, unit)
                except KeyError:
                    tabledat += self._row(p, unit)

        return tabledat

    def prior_summary(self):
        """Summary of priors

        Summarize the priors in separate table within the report PDF.

        Returns:
            string: String containing TeX code for the prior summary table
        """
        
        out = """
\\begin{deluxetable}{lrr}
\\tablecaption{Summary of Priors}
\\tablehead{}
\\startdata
"""
        texdict = self.post.likelihood.params.tex_labels()
        
        prior_list = self.post.priors
        for prior in prior_list:
            out += prior.__str__() + "\\\\\\\\\n"

        out += """
\\enddata
\\end{deluxetable}
"""
        return out
    
    def tex(self, tabtype='all', compstats=None):
        """TeX code for table

        Returns:
            string: TeX code for the results table in the radvel report.
        """
        # Sort extra params
        ep = []
        order = ['gamma', 'dvdt', 'curv', 'jit']
        for o in order:
            op = []
            for p in self.post.likelihood.extra_params:
                if o in p:
                    op.append(p)
            if len(op) == 0: op = [o]
            [ep.append(i) for i in sorted(op)[::-1]]
        ep = ' '.join(ep)

        outstr_params = self._header() + \
                        self._data(self.fitting_basis,
                            sidehead='\\bf{Modified MCMC Step Parameters}')+\
                        self._data(print_basis,
                            sidehead='\\bf{Orbital Parameters}', hline=True)+\
                        self._data(ep,
                            sidehead='\\bf{Other Parameters}', hline=True)+\
                        self._footer()
                        
        if tabtype == 'all':
            outstr = self.tex(tabtype='nplanets', compstats=compstats)+ \
                     outstr_params + \
                     self.prior_summary()
                     
        if tabtype == 'params':
            outstr = outstr_params
            
        if tabtype == 'priors':
            outstr = self.prior_summary()

        if tabtype == 'nplanets':
            outstr = self.comp_table(self.report.compstats)

        # Remove duplicate lines from the
        # step parameters section of the table
        lines = outstr.split('\n')
        trimstr = ""
        for i,line in enumerate(lines):
            if line not in lines[i+1:] or line.startswith('\\') or line == "":
                trimstr += line + "\n"
                 
        return trimstr


    def comp_table(self, statsdict):
        """Model comparisons

        Compare models with increasing number of planets

        Returns:
            string: String containing TeX code for the model comparison table
        """

        if statsdict is None:
            return ""
        
        n_test = range(len(statsdict))

        coldefs = 'r'*len(statsdict)
        
        tstr = """
\\begin{deluxetable*}{l%s}
\\tablecaption{Model Comparison}
\\tablehead{\\colhead{Statistic}""" % coldefs
        for n in n_test:
            if n == max(n_test):
                tstr = tstr + " & \\colhead{{\\bf %d planets (adopted)}}" % n
            else:
                tstr = tstr + " & \\colhead{%d planets}" % n
        tstr += "}\n"
            
        tstr += "\\startdata\n\n"

        statkeys = statsdict[0].keys()
        for s in statkeys:
            row = "%s (%s) " % (s, statsdict[0][s][1])
            for n in n_test:
                row += " & %s" % statsdict[n][s][0]

            row += "\\\\\n"
            tstr += row
        tstr += """
\\enddata
\\label{tab:comp}
\\end{deluxetable*}
"""

        return tstr



import os
import string
import copy

import numpy as np
import pylab as pl
import matplotlib
from mpl_toolkits.axes_grid.anchored_artists import AnchoredText
from mpl_toolkits.axes_grid1 import make_axes_locatable, AxesGrid
from matplotlib.ticker import NullFormatter, MaxNLocator
from matplotlib import rcParams, gridspec
from matplotlib.backends.backend_pdf import PdfPages
from astropy.time import Time
import corner

import radvel
from radvel.utils import t_to_phase, fastbin, round_sig, sigfig

latex = {
    'ms':'m s$^{\mathregular{-1}}$',
    'BJDTDB':'BJD$_{\mathregular{TDB}}$'
}

telfmts_default = {
    'j': dict(color='k',fmt='o',mfc='none',label='HIRES',mew=1),
    'k': dict(color='k',fmt='s',mfc='none',label='HIRES pre 2004',mew=1),
    'a': dict(color='g',fmt='d',label='APF'),
    'pfs': dict(color='magenta',fmt='p',label='PFS'),
    'h': dict(color='firebrick',fmt="s",label='HARPS'),
    'harps-n': dict(color='firebrick',fmt='^',label='HARPS-N'),
    'l': dict(color='g',fmt='*'),
}

telfmts_default['lick'] = telfmts_default['l']
telfmts_default['hires_rj'] = telfmts_default['j']
telfmts_default['hires'] = telfmts_default['j']
telfmts_default['hires_rk'] = telfmts_default['k']
telfmts_default['apf'] = telfmts_default['a']
telfmts_default['harps'] = telfmts_default['h']

cmap = matplotlib.cm.nipy_spectral
rcParams['font.size'] = 9
rcParams['lines.markersize'] = 5
rcParams['axes.grid'] = False
    
def _mtelplot(x, y, e, tel, ax, telfmts={}):
    """Plot data from from multiple telescopes

    x (array): Either time or phase
    y (array): RV
    e (array): RV error
    tel (array): telecsope string key
     telfmts (dict): dictionary of dictionaries corresponding to kwargs 
        passed to errorbar. Example:

        telfmts = {
             'hires': dict(fmt='o',label='HIRES',msize=),
             'harps-n' dict(fmt='s',)}  
    
    """

    lw = 0.5 * rcParams['lines.linewidth']

    default_colors = ['orange', 'purple', 'magenta' , 'pink']
    ci = 0
    
    utel = np.unique(tel)
    for t in utel:
        xt = x[tel == t]
        yt = y[tel == t]
        et = e[tel == t]

        # Default formatting
        kw = dict(
            fmt='o', capsize=0, mew=0, 
            ecolor='0.6', lw = lw, color=default_colors[ci],
            label = t
        )

        # If not explicit format set, look among default formats
        telfmt = {}
        if not telfmts.has_key(t) and telfmts_default.has_key(t):
            telfmt = telfmts_default[t]
        if telfmts.has_key(t):
            telfmt = telfmts[t]
        if not telfmts.has_key(t) and not telfmts_default.has_key(t):
            ci += 1
        for k in telfmt:
            kw[k] = telfmt[k]

        pl.errorbar(xt, yt, yerr=et, **kw)

    ax.yaxis.set_major_formatter(
        matplotlib.ticker.ScalarFormatter(useOffset=False)
    )
    ax.xaxis.set_major_formatter(
        matplotlib.ticker.ScalarFormatter(useOffset=False)
    )

def rv_multipanel_plot(post, saveplot=None, telfmts={}, nobin=False, 
                       yscale_auto=False, yscale_sigma=3.0, nophase=False, 
                       epoch=2450000, uparams=None, phase_ncols=None, 
                       phase_nrows=None, legend=True, rv_phase_space=0.07):
    """Multi-panel RV plot to display model using post.params orbital paramters.

    Args:
        post (radvel.Posterior): Radvel posterior object. The model
            plotted will be generated from post.params
        saveplot (string, optional): Name of output file, will show as
             interactive matplotlib window if not defined.
        nobin (bool, optional): If True do not show binned data on
             phase plots. Will default to True if total number of
             measurements is less then 20.
        yscale_auto (bool, optional): Use matplotlib auto y-axis
             scaling (default: False)
        yscale_sigma (float, optional): Scale y-axis limits to be +/-
             yscale_sigma*(RMS of data plotted)
        telfmts (dict, optional): dictionary of dictionaries mapping
             instrument code to plotting format code.
        nophase (bool, optional): Will omit phase-folded plots if true
        epoch (float, optional): Subtract this value from the time axis for
            more compact axis labels (default: 245000)
        uparams (dict, optional): parameter uncertainties, must
           contain 'per', 'k', and 'e' keys.
        phase_ncols (int, optional): number of columns in the phase
            folded plots. Default behavior is 1.
        phase_nrows (int, optional): number of columns in the phase
            folded plots. Default is nplanets.
        legend (bool, optional): include legend on plot? (default: True)
        rv_phase_space (float, optional): verticle space between rv
            plot and phase-folded plots (in units of fraction of
            figure height)

    Returns:
        figure: current matplotlib figure object
        list: list of axis objects

    """
    figwidth = 7.5 # spans a page with 0.5in margins
    phasefac = 1.5
    ax_rv_height = figwidth * 1/2.
    ax_phase_height = ax_rv_height / phasefac
    bin_fac = 1.75
    bin_markersize = bin_fac * rcParams['lines.markersize']
    bin_markeredgewidth = bin_fac * rcParams['lines.markeredgewidth']
    fit_linewidth = 2.0 * rcParams['lines.linewidth']

    cpspost = copy.deepcopy(post) 
    model = cpspost.likelihood.model
    cpsparams = post.params.basis.to_cps(post.params)
    cpspost.params.update(cpsparams)
    rvtimes = cpspost.likelihood.x
    rvdat = cpspost.likelihood.y
    rverr = cpspost.likelihood.errorbars()
    num_planets = model.num_planets

    if nophase:
        num_planets = 1
        periods = [max(rvtimes) - min(rvtimes)]

    if phase_ncols is None:
        phase_ncols = 1
    if phase_nrows is None:
        phase_nrows = num_planets

    ax_phase_height /= phase_ncols
        
    e = epoch
    if len(post.likelihood.x) < 20: 
        nobin = True
    
    if saveplot != None: 
        resolution = 10000
    else: 
        resolution = 2000


    if isinstance(cpspost.likelihood, radvel.likelihood.CompositeLikelihood):
        like_list = cpspost.likelihood.like_list
    else:
        like_list = [ cpspost.likelihood ]
    
    if not nophase:
        periods = []
        for i in range(num_planets):
            periods.append(cpsparams['per%d' % (i+1)])
            
    longp = max(periods)
    shortp = min(periods)
        
    dt = max(rvtimes) - min(rvtimes)
    rvmodt = np.linspace(
        min(rvtimes) - 0.05 * dt, max(rvtimes) + 0.05 * dt + longp, int(resolution)
    )
    
    rvmod2 = model(rvmodt)
    rvmod = model(rvtimes)

    if ((rvtimes - e) < -2.4e6).any():
        plttimes = rvtimes
        mplttimes = rvmodt
    elif e == 0:
        e = 2450000
        plttimes = rvtimes - e
        mplttimes = rvmodt - e
    else:
        plttimes = rvtimes - e
        mplttimes = rvmodt - e

    rawresid = cpspost.likelihood.residuals()
    resid = (
        rawresid + cpsparams['dvdt']*(rvtimes-model.time_base) 
        + cpsparams['curv']*(rvtimes-model.time_base)**2
    )
    slope = (
        cpsparams['dvdt'] * (rvmodt-model.time_base) 
        + cpsparams['curv'] * (rvmodt-model.time_base)**2
    )
    slope_low = (
        cpsparams['dvdt'] * (rvtimes-model.time_base) 
        + cpsparams['curv'] * (rvtimes-model.time_base)**2
    )


    # Provision figure
    figheight = ax_rv_height + ax_phase_height * phase_nrows
    divide = 1 - ax_rv_height / figheight
    fig = pl.figure(figsize=(figwidth,figheight))
    fig.subplots_adjust(left=0.1)
    gs_rv = gridspec.GridSpec(1, 1)
    gs_rv.update(top=0.95,bottom=divide+rv_phase_space*0.5)
    gs_phase = gridspec.GridSpec(phase_nrows, phase_ncols)
    if phase_ncols==1:
        gs_phase.update(top=divide-rv_phase_space*0.5, bottom=0.07,hspace=0.001)
    else:
        gs_phase.update(top=divide-rv_phase_space*0.5, bottom=0.07,hspace=0.25,wspace=0.25)

    axL = []
    axRV = pl.subplot(gs_rv[0, 0])
    plotindex = 1
    pltletter = ord('a')
    ax = axRV

    axL += [axRV]
   
    #Unphased plot
    ax.axhline(0, color='0.5', linestyle='--')
    ax.plot(mplttimes,rvmod2,'b-', rasterized=False)

    def labelfig(ax, pltletter):
        text = "{})".format(chr(pltletter))
        add_anchored(
            text,loc=2,prop=dict(fontweight='bold',size='large'),frameon=False
        )

    labelfig(ax,pltletter)

    pltletter += 1
    _mtelplot(
        plttimes,rawresid+rvmod,rverr,cpspost.likelihood.telvec, ax, telfmts
    )
    ax.set_xlim(min(plttimes)-0.01*dt,max(plttimes)+0.01*dt)
    
    pl.setp(axRV.get_xticklabels(), visible=False)

    # Legend
    if legend:
        pl.legend(numpoints=1, fontsize='x-small', loc='best')
    
    # Years on upper axis
    axyrs = axRV.twiny()
#    axyrs.set_xlim(min(plttimes)-0.01*dt,max(plttimes)+0.01*dt)


    xl = np.array(list(ax.get_xlim())) + e
    decimalyear = Time(xl,format='jd',scale='utc').decimalyear
    axyrs.plot(decimalyear,decimalyear)
    axyrs.get_xaxis().get_major_formatter().set_useOffset(False)
    axyrs.set_xlim(*decimalyear)
    axyrs.set_xlabel('Year', fontweight='bold')
    #axyrs.xaxis.set_major_locator(MaxNLocator(8))

    if not yscale_auto: 
        scale = np.std(rawresid+rvmod)
        ax.set_ylim(-yscale_sigma * scale , yscale_sigma * scale)

    ax.set_ylabel('RV [{ms:}]'.format(**latex), weight='bold')
    ticks = ax.yaxis.get_majorticklocs()
    ax.yaxis.set_ticks(ticks[1:])

    divider = make_axes_locatable(axRV)
    axResid = divider.append_axes(
        "bottom",size="50%",pad=0.0,sharex=axRV,sharey=None
    )
    ax = axResid
    axL += [axResid]

    #Residuals
    ax.plot(mplttimes,slope,'b-',lw=fit_linewidth)

    labelfig(ax,pltletter)

    pltletter += 1

    _mtelplot(plttimes,resid,rverr, cpspost.likelihood.telvec,ax, telfmts)
    if not yscale_auto: 
        scale = np.std(resid)
        ax.set_ylim(-yscale_sigma * scale, yscale_sigma * scale)

    ax.set_xlim(min(plttimes)-0.01*dt,max(plttimes)+0.01*dt)
    ax.yaxis.set_ticks([ticks[0],0.0,ticks[-1]])
    xticks = ax.xaxis.get_majorticklocs()
    pl.xlabel('JD - {:d}'.format(int(np.round(e))), weight='bold')
    ax.set_ylabel('Residuals', weight='bold')
    ax.yaxis.set_major_locator(MaxNLocator(5,prune='both'))
    
    # Define the locations for the axes
    axbounds = ax.get_position().bounds
    bottom = axbounds[1]
    height = (bottom - 0.15) / num_planets
    textloc = bottom / 2
    bottom -= height + 0.05
    left, width = 0.10, 0.72

    #Phase plots
    for i in range(num_planets):
        if nophase: break
        
        pnum = i+1
        #print "Planet %d" % pnum

        rvdat = rvdat.copy()
        rvmod2 = model(rvmodt, planet_num=pnum) - slope
        modph = t_to_phase(cpspost.params, rvmodt, pnum, cat=True) - 1
        rvdat = rawresid + model(rvtimes, planet_num=pnum) - slope_low
        phase = t_to_phase(cpspost.params, rvtimes, pnum, cat=True) - 1
        p2 = t_to_phase(cpspost.params, rvtimes, pnum, cat=False) - 1
        rvdatcat = np.concatenate((rvdat,rvdat))
        rverrcat = np.concatenate((rverr,rverr))
        rvmod2cat = np.concatenate((rvmod2,rvmod2))
        bint, bindat, binerr = fastbin(phase+1, rvdatcat, nbins=25)
        bint -= 1.0

        i_row = i / phase_ncols
        i_col = i - i_row * phase_ncols
        ax = pl.subplot(gs_phase[i_row, i_col])
        axL += [ax]

        ax.axhline(0, color='0.5', linestyle='--', )
        ax.plot(sorted(modph),rvmod2cat[np.argsort(modph)],'b-',linewidth=fit_linewidth)
        labelfig(ax,pltletter)

        pltletter += 1
        telcat = np.concatenate((cpspost.likelihood.telvec,cpspost.likelihood.telvec))

        _mtelplot(phase,rvdatcat, rverrcat, telcat, ax, telfmts)
        if not nobin and len(rvdat) > 10: 
            ax.errorbar(
                bint, bindat, yerr=binerr, fmt='ro',mec='w', ms=bin_markersize, 
                mew=bin_markeredgewidth
            )

        pl.xlim(-0.5,0.5)

        if not yscale_auto: 
            scale = np.std(rvdatcat)
            pl.ylim(-yscale_sigma*scale, yscale_sigma*scale )
        
        letters = string.lowercase
        planetletter = letters[i+1]
        keys = [p+str(pnum) for p in ['per', 'k', 'e'] ]
        labels = [cpspost.params.tex_labels().get(k, k) for k in keys]
        if i < num_planets-1:
            ticks = ax.yaxis.get_majorticklocs()
            ax.yaxis.set_ticks(ticks[1:-1])

        pl.ylabel('RV [{ms:}]'.format(**latex), weight='bold')
        pl.xlabel('Phase', weight='bold')

        print_params = ['per', 'k', 'e']
        units = {'per':'days','k':latex['ms'],'e':''}

        anotext = []
        for l, p in enumerate(print_params):
            val = cpsparams["%s%d" % (print_params[l],pnum)]
            
            if uparams is None:
                _anotext = '$\\mathregular{%s}$ = %4.2f %s' % (labels[l].replace("$",""), val, units[p])
            else:
                if hasattr(post, 'medparams'):
                    val = post.medparams["%s%d" % (print_params[l],pnum)]
                else:
                    print "WARNING: medparams attribute not found in "+ \
                          "posterior object will annotate with "+ \
                          "max-likelihood values and reported uncertainties "+ \
                          "may not be appropriate."
                err = uparams["%s%d" % (print_params[l],pnum)]
                if err > 0:
                    val, err, errlow = radvel.utils.sigfig(val, err)
                    _anotext = '$\\mathregular{%s}$ = %s $\\mathregular{\\pm}$ %s %s' % (labels[l].replace("$",""), val, err, units[p])
                else:
                    _anotext = '$\\mathregular{%s}$ = %4.2f %s' % (labels[l].replace("$",""), val, units[p])

            anotext += [_anotext] 

        anotext = '\n'.join(anotext)
        add_anchored(
            anotext, loc=1, frameon=True, prop=dict(size='large', weight='bold'),
            bbox=dict(ec='none', fc='w', alpha=0.8)
        )

    if saveplot != None:
        pl.savefig(saveplot,dpi=150)
        print "RV multi-panel plot saved to %s" % saveplot
        
    return fig, axL

    
def corner_plot(post, chains, saveplot=None):
    """
    Make a corner plot from the output MCMC chains and a posterior object.

    Args:
        post (radvel.Posterior): Radvel posterior object
        chains (DataFrame): MCMC chains output by radvel.mcmc
        saveplot (str, optional):  Name of output file, will show as 
            interactive matplotlib window if not defined.

    Returns:
        None
    
    """
    labels = [k for k in post.vary.keys() if post.vary[k]]
    texlabels = [post.params.tex_labels().get(l, l) for l in labels]
    
    f = rcParams['font.size']
    rcParams['font.size'] = 12
    
    fig = corner.corner(
        chains[labels], labels=texlabels, label_kwargs={"fontsize": 14},
        plot_datapoints=False, bins=20, quantiles=[.16,.5,.84],
        show_titles = True, title_kwargs={"fontsize": 14}, smooth=True
    )
    
    if saveplot != None:
        pl.savefig(saveplot,dpi=150)
        print "Corner plot saved to %s" % saveplot
    else:
        pl.show()

    rcParams['font.size'] = f

def texlabel(key, letter):
    if key.count('mpsini')==1:
        return '$M_' + letter + '\\sin i$'
    if key.count('rhop')==1:
        return '$\\rho_' + letter + '$'
        
def corner_plot_derived_pars(chains, P, saveplot=None):
    """
    Make a corner plot from the output MCMC chains and a posterior object.

    Args:
        chains (DataFrame): MCMC chains output by radvel.mcmc
        pars (list): 
        saveplot (Optional[string]: Name of output file, will show as 
            interactive matplotlib window if not defined.

    Returns:
        None
    
    """

    # Determine which columns to include in corner plot
    labels = []
    texlabels = []
    for i in np.arange(1, P.nplanets +1, 1):
        letter = P.planet_letters[i]

        for key in 'mpsini rhop'.split():
            label = '{}{}'.format(key,i)
            
            is_column = list(chains.columns).count(label)==1
            if not is_column:
                break
            
            null_column = chains.isnull().any().ix[label]
            if null_column:
                break
                
            labels.append(label)
            texlabels.append(texlabel(label,letter))

    f = rcParams['font.size']
    rcParams['font.size'] = 12
    fig = corner.corner(
        chains[labels], labels=texlabels, label_kwargs={"fontsize": 14}, 
        plot_datapoints=False, bins=20, quantiles=[0.16,0.50,0.84],
        show_titles = True, title_kwargs={"fontsize": 14}, smooth=True
    )
    
    if saveplot != None:
        pl.savefig(saveplot,dpi=150)
        print "Corner plot saved to %s" % saveplot
    else: pl.show()
    rcParams['font.size'] = f
    
def trend_plot(post, chains, nwalkers, outfile=None):
    """MCMC trend plot

    Make a trend plot to show the evolution of the MCMC as a function of step number.

    Args:
        post (radvel.Posterior): Radvel Posterior object
        chains (DataFrame): MCMC chains output by radvel.mcmc
        nwalkers (int): number of walkers used in this particular MCMC run
        outfile (string): name of output multi-page PDF file

    Returns:
        None
        
    """

    labels = sorted([k for k in post.vary.keys() if post.vary[k]])
    texlabels = [post.params.tex_labels().get(l, l) for l in labels]
    colors = [ cmap(x) for x in np.linspace(0.05, 0.95, nwalkers)]

    quantiles = chains.quantile([0.1587, 0.5, 0.8413])

    with PdfPages(outfile) as pdf:
        for param,tex in zip(labels,texlabels):
            flatchain = chains[param].values
            wchain = flatchain.reshape((nwalkers,-1))
        
        
            fig = pl.figure(figsize=(18,10))
            for w in range(nwalkers):
                pl.plot(
                    wchain[w,:], '.', rasterized=True, color=colors[w], 
                    markersize=3
                )

            pl.xlim(0,wchain.shape[1])

            pl.xlabel('Step Number')
            pl.ylabel(tex)

            ax = pl.gca()
            ax.set_rasterized(True)

            pdf.savefig()
            pl.close()

def add_anchored(*args,**kwargs):
    """
    Parameters
    ----------
    s : string
        Text.

    loc : str
        Location code.

    pad : float, optional
        Pad between the text and the frame as fraction of the font
        size.

    borderpad : float, optional
        Pad between the frame and the axes (or *bbox_to_anchor*).

    prop : `matplotlib.font_manager.FontProperties`
        Font properties.
    """

    bbox = {}
    if kwargs.has_key('bbox'):
        bbox = kwargs.pop('bbox')
    at = AnchoredText(*args, **kwargs)
    if len(bbox.keys())>0:
        pl.setp(at.patch,**bbox)

    ax = pl.gca()
    ax.add_artist(at)



from setuptools import setup, find_packages, Extension
from Cython.Build import cythonize
import numpy
import re

def get_property(prop, project):
    result = re.search(r'{}\s*=\s*[\'"]([^\'"]*)[\'"]'.format(prop), open(project + '/__init__.py').read())
    return result.group(1)

extensions = [
    Extension("radvel._kepler", ["src/_kepler.pyx"],)
]

setup(
    name = "radvel",
    version = get_property('__version__', 'radvel'),
    author = "BJ Fulton, Erik Petigura",
    packages = find_packages(),
    ext_modules = cythonize(extensions),
    include_dirs = [numpy.get_include()],
    data_files = [
        (
            'radvel_example_data', 
            [
                'example_data/164922_fixed.txt', 
                'example_data/epic203771098.csv'
            ]
        )
    ],
    entry_points = {'console_scripts': ['radvel=radvel.cli:main']}
)

print(find_packages())



import numpy as np
import copy_reg
import types

from . import kepler
from .basis import Basis

texdict = {
    'per': 'P',
    'logk': '\\ln{K}',
    'tc': 'T\\rm{conj}',
    'secosw': '\\sqrt{e}\\cos{\\omega}',
    'sesinw': '\\sqrt{e}\\sin{\\omega}',
    'e': 'e',
    'w': '\\omega',
    'tp': 'T\\rm{peri}',
    'k': 'K',
    'gamma_': '\\gamma_{\\rm ',
    'logjit_': '\\ln{\\sigma_{\\rm jit}}_{\\rm ',
    'jit_': '\\sigma_{\\rm ',
    'dvdt': '\\dot{\\gamma}',
    'curv': '\\ddot{\\gamma}'
}

class RVParameters(dict):
    """Object to store the orbital parameters.

    Args:
        num_planets (int): Number of planets in model
        basis (string): parameterization of orbital parameters. See 
            ``radvel.basis.Basis`` for a list of valid basis strings.
        planet_letters (Dictionary[optional): custom map to match the planet 
            numbers in the RVParameter object to planet letters.
            Default {1: 'b', 2: 'c', etc.}. The keys of this dictionary must 
            all be integers.

    Attributes:
        basis (radvel.Basis): Basis object
        planet_parameters (list): orbital parameters contained within the 
            specified basis
        num_planets (int): number of planets in the model
    
    Examples:
       >>> import radvel
       # create a RVParameters object for a 2-planet system with
       # custom planet number to letter mapping
       >>> params = radvel.RVParameters(2, planet_letters={1:'d', 2:'e'})

    """
    def __init__(self, num_planets, basis='per tc secosw sesinw logk', 
                 planet_letters=None):
        self.basis = Basis(basis,num_planets)
        self.planet_parameters = basis.split()
        for num_planet in range(1,1+num_planets):
            for parameter in self.planet_parameters:
                self.__setitem__(self._sparameter(parameter, num_planet), None)
                
        self.num_planets = num_planets
        if planet_letters is not None:
            for k in planet_letters.keys():
                assert isinstance(k, int), "RVParameters: ERROR: The planet_letters dictionary should have only integers as keys."

        self.planet_letters = planet_letters

    def tex_labels(self, param_list=None):
        """Map RVParameters keys to pretty TeX code representations.

        Args:
            param_list (list): (optional) Manually pass a list of parameter labels
        
        Returns:
            dict: dictionary mapping RVParameters keys to TeX code

        """

        if param_list is None:
            param_list = self.keys()
        
        tex_labels = {}
        for k in param_list:
            n = k[-1]
            p = k[:-1]
            if n.isdigit(): tex_labels[k] = self._planet_texlabel(p, n)
            elif k in texdict.keys(): tex_labels[k] = "$%s$" % texdict[k]
            elif p not in self.planet_parameters:
                for tex in texdict.keys():
                    if tex in k and len(tex) > 1:
                        tex_labels[k] = "$%s}$" % k.replace(tex, texdict[tex])
                        
            if k not in tex_labels.keys():
                tex_labels[k] = k

        return tex_labels
        
    def _sparameter(self, parameter, num_planet):
        return '{0}{1}'.format(parameter, num_planet)

    def _planet_texlabel(self, parameter, num_planet):        
        pname = texdict.get(parameter, parameter)
        if self.planet_letters is not None:
            lett_planet = self.planet_letters[int(num_planet)]
        else:
            lett_planet = chr(int(num_planet)+97)
        return '$%s_{%s}$' % (pname, lett_planet) 

class RVModel(object):
    """
    Generic RV Model

    This class defines the methods common to all RV modeling
    classes. The different RV models, having different
    parameterizations inherit from this class.
    """
    def __init__(self, params, time_base=0):
        self.num_planets = params.num_planets
        self.params = params
        self.params['dvdt'] = 0
        self.params['curv'] = 0
        self.time_base = time_base

    def __call__(self, t, planet_num=None):
        """Compute the radial velocity
        
        Includes all Keplerians and additional trends.

        Args:
            t (array of floats): Timestamps to calculate the RV model
            planet_num (Optional[int]): calculate the RV model for a single 
                planet within a multi-planet system

        Returns:
            vel (array of floats): Radial velocity at each time in `t`
        """
        vel = np.zeros( len(t) )
        params_cps = self.params.basis.to_cps(self.params)

        if planet_num == None:
            planets = range(1, self.num_planets+1)
        else:
            planets = [planet_num]
        
        for num_planet in planets:
            per = params_cps['per{}'.format(num_planet)]
            tp = params_cps['tp{}'.format(num_planet)]
            e = params_cps['e{}'.format(num_planet)]
            w = params_cps['w{}'.format(num_planet)]
            k = params_cps['k{}'.format(num_planet)]
            orbel_cps = np.array([per, tp, e, w, k])
            vel+=kepler.rv_drive(t, orbel_cps)

        vel+=self.params['dvdt'] * ( t - self.time_base )
        vel+=self.params['curv'] * ( t - self.time_base )**2
        return vel

# I had to add these methods to get the model object to be
# pickle-able, so we could run the mcmc in as a in multi-threaded
# mode.
def _pickle_method(method):
    func_name = method.im_func.__name__
    obj = method.im_self
    cls = method.im_class
    return _unpickle_method, (func_name, obj, cls)

def _unpickle_method(func_name, obj, cls):
    for cls in cls.mro():
        try:
            func = cls.__dict__[func_name]
        except KeyError:
            pass
        else:
            break
    return func.__get__(obj, cls)
copy_reg.pickle(types.MethodType, _pickle_method, _unpickle_method)
